__code__='Gas1]?ZX[c&Dr[^a"_:I9hPFO;5(A\\Te[dUl#\\tW1+!]I\'K:YqFA+b77=.ToA4`<:M5(AQ@AY^V,5iGY(jbSYkYl[?Mn%-MF165)g\\q(o=80>A@IA>nk)V@j^W>=j(]jPf!YF9)YWX^K,X;\\r!FAHL#AF1d0EEc@#%/*`!O)a;#hT>D#Ijp3*oJg3H,\'pJqg_tf$\\0aX*roUMr"/?B^lJL\\!W3Z$\'&[?:cN(-[bPC<JF5?`\\\')YAW(5s.SS",&:5W7Q="S(I^fQq/G1Ldfo5Nc!9Pf$K<3a\\+\\/"A9>X:+)]AQ8ku251GrXH$?`7OJ-U3gLE>2$hH0YXPlaZd/UmG=]NSJ;Z\'TciD*C9Eq2\\nctgV]jk)GC\'0J*aR7W^Jt%$05U9?BM/?(Zo<7?Vk+d]aa6WKf5Bb7k@9;_ZCK3"JZUH*;;5,)3ZuR&QPThR$EP=Li[Z`pA[4%r&*#6Yr<l*C\'m](V\\Xd?)m2r!UlF;3oOa`\\StRiRb`Q>pb%a+7o^Y3:\\Yg2aNY#[4s7"Cp]nA_Zt".8F>Y0DYU*8J%McW&DUpXE/!9%6)R?BJrLmADS=:M[fQGbYk:N@,\'Q1Sdb!7[f\\^P6CHmIm7\\*`W9n\\>c>QMEm(6TmnF4e)eRfe;]4!aDD8I9;KtDdF`03GVBk"!_QI@B_KGCB;d=6_7OFf"jnqnCb<2iHdC[u]HpC"V,J#D0nEUA_e&#@?.1S(DOP!l-B>rTFUl<N=)e@.]B_Th(@5>?5bI5sC\'nQ%^kU9Lt\\$MT-W\\eMY/(bOXV\\U\'IZS^hXrhc]iB2kLf?1R*ZOQ7<2M8ne0iIdpu+k*%S>:;WLn-ck].P$n,md0]PX\\0!$/FU)X8>;*R15O7cW4XY?TJ%o*2h.[A+GCh6/a?`$,mJ86ekBld,kRM/`;3=&JT13O"[HP#Sq-l3s%D$)I?a6>sMs?:H$,ASH>>(,JB$_LEL94\'Z@PF!\'hf"H?9t-L4Xn\\VOU(l\'8Y&`4f/\'2(/6E<_P,?0F?#"MW`:9.Xhr?.+,W^=-2[qnH9Nq608Qf:98bc7?9;bPg?r`!5hE-T+@-&N)T7:C@[Yp,XV^I,MB$3~>'



#copyright ReportLab Europe Limited. 2000-2012
#see license.txt for license details
'''
RC4 Stream  ciphering
'''
__version__=''' $Id$ '''

from reportlab.lib.arciv import ArcIV as RC4, _TESTS

if __name__=='__main__':
	i = 0
	for t in _TESTS:
		o = RC4(t['key']).encode(t['input'])
		print('Forward test %d %s!' %(i,o!=t['output'] and 'failed' or 'succeeded'))
		o = RC4(t['key']).encode(t['output'])
		print('Reverse test %d %s!' %(i,o!=t['input'] and 'failed' or 'succeeded'))
		i += 1



# vim: sw=4:expandtab:foldmethod=marker
#
# Copyright (c) 2006, Mathieu Fenniak
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# * Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# * The name of the author may not be used to endorse or promote products
# derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""
Utility functions for PDF library.
"""
__author__ = "Mathieu Fenniak"
__author_email__ = "biziqe@mathieu.fenniak.net"

#custom implementation of warnings.formatwarning 
def _formatwarning(message, category, filename, lineno, line=None):
    file = filename.replace("/", "\\").rsplit("\\", 1)[1] # find the file name
    return "%s: %s [%s:%s]\n" % (category.__name__, message, file, lineno)

def readUntilWhitespace(stream, maxchars=None):
    """
    Reads non-whitespace characters and returns them.
    Stops upon encountering whitespace or when maxchars is reached.
    """
    txt = b_("")
    while True:
        tok = stream.read(1)
        if tok.isspace() or not tok:
            break
        txt += tok
        if len(txt) == maxchars:
            break
    return txt

def readNonWhitespace(stream):
    """
    Finds and reads the next non-whitespace character (ignores whitespace).
    """
    tok = b_(' ')
    while tok == b_('\n') or tok == b_('\r') or tok == b_(' ') or tok == b_('\t'):
        tok = stream.read(1)
    return tok

def skipOverWhitespace(stream):
    """
    Similar to readNonWhitespace, but returns a Boolean if more than
    one whitespace character was read.
    """
    tok = b_(' ')
    cnt = 0;
    while tok == b_('\n') or tok == b_('\r') or tok == b_(' ') or tok == b_('\t'):
        tok = stream.read(1)
        cnt+=1
    return (cnt > 1)

def skipOverComment(stream):
    tok = stream.read(1)
    stream.seek(-1, 1)
    if tok == b_('%'):
        while tok not in (b_('\n'), b_('\r')):
            tok = stream.read(1)

class ConvertFunctionsToVirtualList(object):
    def __init__(self, lengthFunction, getFunction):
        self.lengthFunction = lengthFunction
        self.getFunction = getFunction

    def __len__(self):
        return self.lengthFunction()

    def __getitem__(self, index):
        if not isinstance(index, int):
            raise TypeError("sequence indices must be integers")
        len_self = len(self)
        if index < 0:
            # support negative indexes
            index = len_self + index
        if index < 0 or index >= len_self:
            raise IndexError("sequence index out of range")
        return self.getFunction(index)

def RC4_encrypt(key, plaintext):
    S = [i for i in range(256)]
    j = 0
    for i in range(256):
        j = (j + S[i] + ord_(key[i % len(key)])) % 256
        S[i], S[j] = S[j], S[i]
    i, j = 0, 0
    retval = b_("")
    for x in range(len(plaintext)):
        i = (i + 1) % 256
        j = (j + S[i]) % 256
        S[i], S[j] = S[j], S[i]
        t = S[(S[i] + S[j]) % 256]
        retval += b_(chr(ord_(plaintext[x]) ^ t))
    return retval

def matrixMultiply(a, b):
    return [[sum([float(i)*float(j)
                  for i, j in zip(row, col)]
                ) for col in zip(*b)]
            for row in a]

def markLocation(stream):
    """Creates text file showing current location in context."""
    # Mainly for debugging
    RADIUS = 5000
    stream.seek(-RADIUS, 1)
    outputDoc = open('PyPDF2_pdfLocation.txt', 'w')
    outputDoc.write(stream.read(RADIUS))
    outputDoc.write('HERE')
    outputDoc.write(stream.read(RADIUS))
    outputDoc.close()
    stream.seek(-RADIUS, 1)

class PyPdfError(Exception):
    pass

class PdfReadError(PyPdfError):
    pass

class PageSizeNotDefinedError(PyPdfError):
    pass
    
class PdfReadWarning(UserWarning):
    pass

class PdfStreamError(PdfReadError):
    pass

def hexStr(num):
    return hex(num).replace('L', '')

import sys

def b_(s):
    if sys.version_info[0] < 3:
        return s
    else:
        if type(s) == bytes:
            return s
        else:
            return s.encode('latin-1')

def u_(s):
    if sys.version_info[0] < 3:
        return unicode(s, 'unicode_escape')
    else:
        return s


def str_(b):
    if sys.version_info[0] < 3:
        return b
    else:
        if type(b) == bytes:
            return b.decode('latin-1')
        else:
            return b

def ord_(b):
    if sys.version_info[0] < 3 or type(b) == str:
        return ord(b)
    else:
        return b

def chr_(c):
    if sys.version_info[0] < 3:
        return c
    else:
        return chr(c)

def barray(b):
    if sys.version_info[0] < 3:
        return b
    else:
        return bytearray(b)

def hexencode(b):
    if sys.version_info[0] < 3:
        return b.encode('hex')
    else:
        import codecs
        coder = codecs.getencoder('hex_codec')
        return coder(b)[0]

if sys.version_info[0] < 3:
    string_type = unicode
    bytes_type = str
else:
    string_type = str
    bytes_type = bytes



#copyright ReportLab Inc. 2000-2012
#see license.txt for license details
#history www.reportlab.co.uk/rl-cgi/viewcvs.cgi/rlextra/__init__.py
Version='3.1.43'

#legacy support, to be removed.
__version__=''' $Id: __init__.py 99999 2013-04-03 15:00:00Z andy $ '''
import sys

if sys.version_info[0:2]!=(2, 7) and sys.version_info<(3, 3):
    raise ImportError("""rlextra requires Python 2.7+ or 3.3+; 3.0-3.2 are not supported.""")

import reportlab
if reportlab.Version < '3.0.0':
	raise ImportError("""rlextra version %s requires at least reportlab 3.0.0, and you have reportlab %s; it is strongly recommended that you use the same versions""" % (Version, reportlab.Version))



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin, String, Rect
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfbase.pdfmetrics import registerFont
from reportlab.lib.colors import red

class TestArialUni(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=800,*args,**kw):
		DataAwareDrawing.__init__(self,width,height,*args,**kw)
		registerFont(TTFont('arialuni','arialuni.ttf'))
		self._add(self,String(10,10,'text \xd0\x96 \xe7\xa4\xba \xe6\xbc\xb9',fontName='arialuni',fontSize=36),name='s0',validate=None,desc=None)
		bb = self.s0.getBounds()
		self._add(self,Rect(bb[0],bb[1],bb[2]-bb[0],bb[3]-bb[1],fillColor=None,strokeWidth=1,strokeColor=red),name=None,validate=None,desc=None)
		self.__dict__['preview'] = 0
		self._add(self,String(10,20,"""u0020=  u0021=! u0022=" u0023=# u0024=$ u0025=% u0026=&""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,32,"""u0027=' u0028=( u0029=) u002a=* u002b=+ u002c=, u002d=- u002e=.""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,44,"""u002f=/ u0030=0 u0031=1 u0032=2 u0033=3 u0034=4 u0035=5 u0036=6""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,56,"""u0037=7 u0038=8 u0039=9 u003a=: u003b=; u003c=< u003d== u003e=>""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,68,"""u003f=? u0040=@ u0041=A u0042=B u0043=C u0044=D u0045=E u0046=F""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,80,"""u0047=G u0048=H u0049=I u004a=J u004b=K u004c=L u004d=M u004e=N""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,92,"""u004f=O u0050=P u0051=Q u0052=R u0053=S u0054=T u0055=U u0056=V""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,104,"""u0057=W u0058=X u0059=Y u005a=Z u005b=[ u005c=\\ u005d=] u005e=^""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,116,"""u005f=_ u0060=` u0061=a u0062=b u0063=c u0064=d u0065=e u0066=f""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,128,"""u0067=g u0068=h u0069=i u006a=j u006b=k u006c=l u006d=m u006e=n""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,140,"""u006f=o u0070=p u0071=q u0072=r u0073=s u0074=t u0075=u u0076=v""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,152,"""u0077=w u0078=x u0079=y u007a=z u007b={ u007c=| u007d=} u007e=~""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,164,"""u00a0=\xc2\xa0 u00a1=\xc2\xa1 u00a2=\xc2\xa2 u00a3=\xc2\xa3 u00a4=\xc2\xa4 u00a5=\xc2\xa5 u00a6=\xc2\xa6 u00a7=\xc2\xa7""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,176,"""u00a8=\xc2\xa8 u00a9=\xc2\xa9 u00aa=\xc2\xaa u00ab=\xc2\xab u00ac=\xc2\xac u00ad=\xc2\xad u00ae=\xc2\xae u00af=\xc2\xaf""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,188,"""u00b0=\xc2\xb0 u00b1=\xc2\xb1 u00b2=\xc2\xb2 u00b3=\xc2\xb3 u00b4=\xc2\xb4 u00b5=\xc2\xb5 u00b6=\xc2\xb6 u00b7=\xc2\xb7""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,200,"""u00b8=\xc2\xb8 u00b9=\xc2\xb9 u00ba=\xc2\xba u00bb=\xc2\xbb u00bc=\xc2\xbc u00bd=\xc2\xbd u00be=\xc2\xbe u00bf=\xc2\xbf""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,212,"""u00c0=\xc3\x80 u00c1=\xc3\x81 u00c2=\xc3\x82 u00c3=\xc3\x83 u00c4=\xc3\x84 u00c5=\xc3\x85 u00c6=\xc3\x86 u00c7=\xc3\x87""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,224,"""u00c8=\xc3\x88 u00c9=\xc3\x89 u00ca=\xc3\x8a u00cb=\xc3\x8b u00cc=\xc3\x8c u00cd=\xc3\x8d u00ce=\xc3\x8e u00cf=\xc3\x8f""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,236,"""u00d0=\xc3\x90 u00d1=\xc3\x91 u00d2=\xc3\x92 u00d3=\xc3\x93 u00d4=\xc3\x94 u00d5=\xc3\x95 u00d6=\xc3\x96 u00d7=\xc3\x97""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,248,"""u00d8=\xc3\x98 u00d9=\xc3\x99 u00da=\xc3\x9a u00db=\xc3\x9b u00dc=\xc3\x9c u00dd=\xc3\x9d u00de=\xc3\x9e u00df=\xc3\x9f""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,260,"""u00e0=\xc3\xa0 u00e1=\xc3\xa1 u00e2=\xc3\xa2 u00e3=\xc3\xa3 u00e4=\xc3\xa4 u00e5=\xc3\xa5 u00e6=\xc3\xa6 u00e7=\xc3\xa7""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,272,"""u00e8=\xc3\xa8 u00e9=\xc3\xa9 u00ea=\xc3\xaa u00eb=\xc3\xab u00ec=\xc3\xac u00ed=\xc3\xad u00ee=\xc3\xae u00ef=\xc3\xaf""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,284,"""u00f0=\xc3\xb0 u00f1=\xc3\xb1 u00f2=\xc3\xb2 u00f3=\xc3\xb3 u00f4=\xc3\xb4 u00f5=\xc3\xb5 u00f6=\xc3\xb6 u00f7=\xc3\xb7""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,296,"""u00f8=\xc3\xb8 u00f9=\xc3\xb9 u00fa=\xc3\xba u00fb=\xc3\xbb u00fc=\xc3\xbc u00fd=\xc3\xbd u00fe=\xc3\xbe u00ff=\xc3\xbf""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,308,"""u0100=\xc4\x80 u0101=\xc4\x81 u0102=\xc4\x82 u0103=\xc4\x83 u0104=\xc4\x84 u0105=\xc4\x85 u0106=\xc4\x86 u0107=\xc4\x87""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,320,"""u0108=\xc4\x88 u0109=\xc4\x89 u010a=\xc4\x8a u010b=\xc4\x8b u010c=\xc4\x8c u010d=\xc4\x8d u010e=\xc4\x8e u010f=\xc4\x8f""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,332,"""u2019=\xe2\x80\x99 u201a=\xe2\x80\x9a u201b=\xe2\x80\x9b u201c=\xe2\x80\x9c u201d=\xe2\x80\x9d u201e=\xe2\x80\x9e u201f=\xe2\x80\x9f u2020=\xe2\x80\xa0""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,344,"""u2021=\xe2\x80\xa1 u2022=\xe2\x80\xa2 u2023=\xe2\x80\xa3 u2024=\xe2\x80\xa4 u2025=\xe2\x80\xa5 u2026=\xe2\x80\xa6 u2027=\xe2\x80\xa7 u2028=\xe2\x80\xa8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,356,"""u2029=\xe2\x80\xa9 u202a=\xe2\x80\xaa u202b=\xe2\x80\xab u202c=\xe2\x80\xac u202d=\xe2\x80\xad u202e=\xe2\x80\xae u202f=\xe2\x80\xaf u2030=\xe2\x80\xb0""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,368,"""u2031=\xe2\x80\xb1 u2032=\xe2\x80\xb2 u2033=\xe2\x80\xb3 u2034=\xe2\x80\xb4 u2035=\xe2\x80\xb5 u2036=\xe2\x80\xb6 u2037=\xe2\x80\xb7 u2038=\xe2\x80\xb8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,380,"""u2039=\xe2\x80\xb9 u203a=\xe2\x80\xba u203b=\xe2\x80\xbb u203c=\xe2\x80\xbc u203d=\xe2\x80\xbd u203e=\xe2\x80\xbe u203f=\xe2\x80\xbf u2040=\xe2\x81\x80""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,392,"""u2041=\xe2\x81\x81 u2042=\xe2\x81\x82 u2043=\xe2\x81\x83 u2044=\xe2\x81\x84 u2045=\xe2\x81\x85 u2046=\xe2\x81\x86 u2047=\xe2\x81\x87 u2048=\xe2\x81\x88""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,452,"""u206f=\xe2\x81\xaf u2070=\xe2\x81\xb0 u2071=\xe2\x81\xb1 u2074=\xe2\x81\xb4 u2075=\xe2\x81\xb5 u2076=\xe2\x81\xb6 u2077=\xe2\x81\xb7 u2078=\xe2\x81\xb8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,464,"""u2079=\xe2\x81\xb9 u207a=\xe2\x81\xba u207b=\xe2\x81\xbb u207c=\xe2\x81\xbc u207d=\xe2\x81\xbd u207e=\xe2\x81\xbe u207f=\xe2\x81\xbf u2080=\xe2\x82\x80""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,476,"""u2081=\xe2\x82\x81 u2082=\xe2\x82\x82 u2083=\xe2\x82\x83 u2084=\xe2\x82\x84 u2085=\xe2\x82\x85 u2086=\xe2\x82\x86 u2087=\xe2\x82\x87 u2088=\xe2\x82\x88""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,488,"""u2089=\xe2\x82\x89 u208a=\xe2\x82\x8a u208b=\xe2\x82\x8b u208c=\xe2\x82\x8c u208d=\xe2\x82\x8d u208e=\xe2\x82\x8e u2090=\xe2\x82\x90 u2091=\xe2\x82\x91""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,500,"""u2092=\xe2\x82\x92 u2093=\xe2\x82\x93 u2094=\xe2\x82\x94 u20a0=\xe2\x82\xa0 u20a1=\xe2\x82\xa1 u20a2=\xe2\x82\xa2 u20a3=\xe2\x82\xa3 u20a4=\xe2\x82\xa4""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,512,"""u20a5=\xe2\x82\xa5 u20a6=\xe2\x82\xa6 u20a7=\xe2\x82\xa7 u20a8=\xe2\x82\xa8 u20a9=\xe2\x82\xa9 u20aa=\xe2\x82\xaa u20ab=\xe2\x82\xab u20ac=\xe2\x82\xac""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,524,"""u20ad=\xe2\x82\xad u20ae=\xe2\x82\xae u20af=\xe2\x82\xaf u20b0=\xe2\x82\xb0 u20b1=\xe2\x82\xb1 u20b2=\xe2\x82\xb2 u20b3=\xe2\x82\xb3 u20b4=\xe2\x82\xb4""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,536,"""u20b5=\xe2\x82\xb5 u20d0=\xe2\x83\x90 u20d1=\xe2\x83\x91 u20d2=\xe2\x83\x92 u20d3=\xe2\x83\x93 u20d4=\xe2\x83\x94 u20d5=\xe2\x83\x95 u20d6=\xe2\x83\x96""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,548,"""u20d7=\xe2\x83\x97 u20d8=\xe2\x83\x98 u20d9=\xe2\x83\x99 u20da=\xe2\x83\x9a u20db=\xe2\x83\x9b u20dc=\xe2\x83\x9c u20dd=\xe2\x83\x9d u20de=\xe2\x83\x9e""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,560,"""u20df=\xe2\x83\x9f u20e0=\xe2\x83\xa0 u20e1=\xe2\x83\xa1 u20e2=\xe2\x83\xa2 u20e3=\xe2\x83\xa3 u20e4=\xe2\x83\xa4 u20e5=\xe2\x83\xa5 u20e6=\xe2\x83\xa6""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,572,"""u20e7=\xe2\x83\xa7 u20e8=\xe2\x83\xa8 u20e9=\xe2\x83\xa9 u20ea=\xe2\x83\xaa u20eb=\xe2\x83\xab u2100=\xe2\x84\x80 u2101=\xe2\x84\x81 u2102=\xe2\x84\x82""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,584,"""u2103=\xe2\x84\x83 u2104=\xe2\x84\x84 u2105=\xe2\x84\x85 u2106=\xe2\x84\x86 u2107=\xe2\x84\x87 u2108=\xe2\x84\x88 u2109=\xe2\x84\x89 u210a=\xe2\x84\x8a""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,596,"""u210b=\xe2\x84\x8b u210c=\xe2\x84\x8c u210d=\xe2\x84\x8d u210e=\xe2\x84\x8e u210f=\xe2\x84\x8f u2110=\xe2\x84\x90 u2111=\xe2\x84\x91 u2112=\xe2\x84\x92""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,608,"""u2113=\xe2\x84\x93 u2114=\xe2\x84\x94 u2115=\xe2\x84\x95 u2116=\xe2\x84\x96 u2117=\xe2\x84\x97 u2118=\xe2\x84\x98 u2119=\xe2\x84\x99 u211a=\xe2\x84\x9a""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,620,"""u211b=\xe2\x84\x9b u211c=\xe2\x84\x9c u211d=\xe2\x84\x9d u211e=\xe2\x84\x9e u211f=\xe2\x84\x9f u2120=\xe2\x84\xa0 u2121=\xe2\x84\xa1 u2122=\xe2\x84\xa2""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,632,"""u2123=\xe2\x84\xa3 u2124=\xe2\x84\xa4 u2125=\xe2\x84\xa5 u2126=\xe2\x84\xa6 u2127=\xe2\x84\xa7 u2128=\xe2\x84\xa8 u2129=\xe2\x84\xa9 u212a=\xe2\x84\xaa""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,644,"""u212b=\xe2\x84\xab u212c=\xe2\x84\xac u212d=\xe2\x84\xad u212e=\xe2\x84\xae u212f=\xe2\x84\xaf u2130=\xe2\x84\xb0 u2131=\xe2\x84\xb1 u2132=\xe2\x84\xb2""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,656,"""u2133=\xe2\x84\xb3 u2134=\xe2\x84\xb4 u2135=\xe2\x84\xb5 u2136=\xe2\x84\xb6 u2137=\xe2\x84\xb7 u2138=\xe2\x84\xb8 u2139=\xe2\x84\xb9 u213a=\xe2\x84\xba""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,668,"""u213b=\xe2\x84\xbb u213c=\xe2\x84\xbc u213d=\xe2\x84\xbd u213e=\xe2\x84\xbe u213f=\xe2\x84\xbf u2140=\xe2\x85\x80 u2141=\xe2\x85\x81 u2142=\xe2\x85\x82""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,680,"""u2143=\xe2\x85\x83 u2144=\xe2\x85\x84 u2145=\xe2\x85\x85 u2146=\xe2\x85\x86 u2147=\xe2\x85\x87 u2148=\xe2\x85\x88 u2149=\xe2\x85\x89 u214a=\xe2\x85\x8a""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,692,"""u214b=\xe2\x85\x8b u214c=\xe2\x85\x8c u2153=\xe2\x85\x93 u2154=\xe2\x85\x94 u2155=\xe2\x85\x95 u2156=\xe2\x85\x96 u2157=\xe2\x85\x97 u2158=\xe2\x85\x98""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,704,"""u2159=\xe2\x85\x99 u215a=\xe2\x85\x9a u215b=\xe2\x85\x9b u215c=\xe2\x85\x9c u215d=\xe2\x85\x9d u215e=\xe2\x85\x9e u215f=\xe2\x85\x9f u2160=\xe2\x85\xa0""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,716,"""u2161=\xe2\x85\xa1 u2162=\xe2\x85\xa2 u2163=\xe2\x85\xa3 u2164=\xe2\x85\xa4 u2165=\xe2\x85\xa5 u2166=\xe2\x85\xa6 u2167=\xe2\x85\xa7 u2168=\xe2\x85\xa8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,728,"""u2169=\xe2\x85\xa9 u216a=\xe2\x85\xaa u216b=\xe2\x85\xab u216c=\xe2\x85\xac u216d=\xe2\x85\xad u216e=\xe2\x85\xae u216f=\xe2\x85\xaf u2170=\xe2\x85\xb0""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,740,"""u2171=\xe2\x85\xb1 u2172=\xe2\x85\xb2 u2173=\xe2\x85\xb3 u2174=\xe2\x85\xb4 u2175=\xe2\x85\xb5 u2176=\xe2\x85\xb6 u2177=\xe2\x85\xb7 u2178=\xe2\x85\xb8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)

if __name__=="__main__": #NORUNTESTS
	TestArialUni().save(formats=['pdf','eps','gif'],outDir='.',fnRoot=None,preview=0)



#tests pdf image
from rlextra.pageCatcher import pageCatcher
from reportlab.pdfgen.canvas import Canvas
def run():
    c = Canvas('test_pdf_image.pdf')
    c.drawString(72, 720, "Testing PDFs as images....")
    c.drawString(72, 700, "These all use a blue-ish vector icon of a PC")

    imageName = "internet_access.pdf"

    #natural size
    c.drawString(72, 680, "Huge image off the right edge ---->")
    pageCatcher.drawPdfImage(imageName, c),


    c.drawString(5, 25, "little 25x25 point image at page corner")

    pageCatcher.drawPdfImage(imageName, c,
                             width=25, height=25,
                             showBoundary=True)

    #move it a bit
    c.drawString(100, 630, "50x25 image below me, origin (600,100)...")
    pageCatcher.drawPdfImage(imageName, c,
                             x=100, y=600,
                             width=50, height=25,
                             showBoundary=True)


    #preserve aspect 
    c.drawString(100, 530, "zoomed to fit at west and east edges of 50x25 box")
    pageCatcher.drawPdfImage(imageName, c,
                             x=100, y=500,
                             width=50, height=25,
                             showBoundary=True,
                             preserveAspectRatio=True
                             )
    pageCatcher.drawPdfImage(imageName, c,
                             x=200, y=500,
                             width=50, height=25,
                             showBoundary=True,
                             preserveAspectRatio=True,
                             anchor="e"
                             )
    c.drawString(100, 460, "zoomed to fit at north and south edges of 25x50 box")

    pageCatcher.drawPdfImage(imageName, c,
                             x=100, y=400,
                             width=25, height=50,
                             showBoundary=True,
                             preserveAspectRatio=True,
                             anchor='n'
                             )
    pageCatcher.drawPdfImage(imageName, c,
                             x=200, y=400,
                             width=25, height=50,
                             showBoundary=True,
                             preserveAspectRatio=True,
                             anchor='s'
                             )


    #aspect ratio should be kept if shrinking...


##def drawPdfImage(fileName, canv, x=0, y=0, width=None, height=None,
##                 preserveAspectRatio=False, pageNumber=0):


    c.save()
    print('saved test_pdf_image.pdf')

if __name__=='__main__':
    run()



#!/usr/bin/env python
"""
Representation and utils for ranges of PDF file pages.

Copyright (c) 2014, Steve Witham <switham_github@mac-guyver.com>.
All rights reserved. This software is available under a BSD license;
see https://github.com/mstamy2/PyPDF2/LICENSE
"""

import re

# "Str" maintains compatibility with Python 2.x.
# The next line is obfuscated like this so 2to3 won't change it.
Str = getattr(__builtins__, "basestring", str)

_INT_RE = r"(0|-?[1-9]\d*)"  # A decimal int, don't allow "-0".
PAGE_RANGE_RE = "^({int}|({int}?(:{int}?(:{int}?)?)))$".format(int=_INT_RE)
# groups:         12     34     5 6     7 8


class ParseError(Exception):
    pass


PAGE_RANGE_HELP = """Remember, page indices start with zero.
        Page range expression examples:
            :     all pages.                   -1    last page.
            22    just the 23rd page.          :-1   all but the last page.
            0:3   the first three pages.       -2    second-to-last page.
            :3    the first three pages.       -2:   last two pages.
            5:    from the sixth page onward.  -3:-1 third & second to last.
        The third, "stride" or "step" number is also recognized.
            ::2       0 2 4 ... to the end.    3:0:-1    3 2 1 but not 0.
            1:10:2    1 3 5 7 9                2::-1     2 1 0.
            ::-1      all pages in reverse order.
"""

        
class PageRange(object):
    """ 
    A slice-like representation of a range of page indices,
        i.e. page numbers, only starting at zero. 
    The syntax is like what you would put between brackets [ ].
    The slice is one of the few Python types that can't be subclassed,
    but this class converts to and from slices, and allows similar use.
      o  PageRange(str) parses a string representing a page range.
      o  PageRange(slice) directly "imports" a slice.
      o  to_slice() gives the equivalent slice.
      o  str() and repr() allow printing.
      o  indices(n) is like slice.indices(n).
    """
    
    def __init__(self, arg):
        """
        Initialize with either a slice -- giving the equivalent page range,
        or a PageRange object -- making a copy,
        or a string like
            "int", "[int]:[int]" or "[int]:[int]:[int]",
            where the brackets indicate optional ints.
        {page_range_help}
        Note the difference between this notation and arguments to slice():
            slice(3) means the first three pages;
            PageRange("3") means the range of only the fourth page.
            However PageRange(slice(3)) means the first three pages.
        """
        if isinstance(arg, slice):
            self._slice = arg
            return

        if isinstance(arg, PageRange):
            self._slice = arg.to_slice()
            return
        
        m = isinstance(arg, Str) and re.match(PAGE_RANGE_RE, arg)
        if not m:
            raise ParseError(arg)
        elif m.group(2):
            # Special case: just an int means a range of one page.
            start = int(m.group(2))
            stop = start + 1 if start != -1 else None
            self._slice = slice(start, stop)
        else:
            self._slice = slice(*[int(g) if g else None 
                                  for g in m.group(4, 6, 8)])

    __init__.__doc__ = __init__.__doc__.format(page_range_help=PAGE_RANGE_HELP)
        
    @staticmethod
    def valid(input):
        """ True if input is a valid initializer for a PageRange. """
        return isinstance(input, slice)  or \
               isinstance(input, PageRange) or \
               (isinstance(input, Str)
                and bool(re.match(PAGE_RANGE_RE, input)))

    def to_slice(self):
        """ Return the slice equivalent of this page range. """
        return self._slice
        
    def __str__(self):
        """ A string like "1:2:3". """
        s = self._slice
        if s.step == None:
            if s.start != None  and  s.stop == s.start + 1:
                return str(s.start)

            indices = s.start, s.stop
        else:
            indices = s.start, s.stop, s.step
        return ':'.join("" if i == None else str(i) for i in indices)

    def __repr__(self):
        """ A string like "PageRange('1:2:3')". """
        return "PageRange(" + repr(str(self)) + ")"

    def indices(self, n):
        """
        n is the length of the list of pages to choose from.
        Returns arguments for range().  See help(slice.indices).
        """
        return self._slice.indices(n)


PAGE_RANGE_ALL = PageRange(":")  # The range of all pages.


def parse_filename_page_ranges(args):
    """
    Given a list of filenames and page ranges, return a list of
    (filename, page_range) pairs.
    First arg must be a filename; other ags are filenames, page-range 
    expressions, slice objects, or PageRange objects.
    A filename not followed by a page range indicates all pages of the file.
    """
    pairs = []
    pdf_filename = None
    did_page_range = False
    for arg in args + [None]:
        if PageRange.valid(arg):
            if not pdf_filename:
                raise ValueError("The first argument must be a filename, " \
                                 "not a page range.")

            pairs.append( (pdf_filename, PageRange(arg)) )
            did_page_range = True
        else:
            # New filename or end of list--do all of the previous file?
            if pdf_filename and not did_page_range:
                pairs.append( (pdf_filename, PAGE_RANGE_ALL) )
                    
            pdf_filename = arg
            did_page_range = False
    return pairs



#copyright ReportLab Europe Limited. 2000-2012
#see license.txt for license details
if __name__=='__main__': #no run tests
	import os, shutil, sys
	try:
		import Installer
		INSTALLER_DIR=os.path.dirname(Installer.__file__)
	except:
		import rlextra
		INSTALLER_DIR=os.path.join(os.path.dirname(rlextra.__file__),'distro\\tools\\Installer')

	BUILD_DIR='build_freeze'
	NAME='rml2pdf'
	EXE=NAME+'.exe'
	SCRIPT='exemain.py'
	CFG=NAME+'.cfg'
	DEBUG="-d" in sys.argv
	PDEBUG="-pd" in sys.argv

	#these are forced into the EXE
	MISC_DLLS='' #'ws2_32.dll, ws2help.dll'

	def bpath(fn):
		return os.path.join(BUILD_DIR,fn)
	def ipath(fn):
		return os.path.join(INSTALLER_DIR,fn)
	PIL_names=['Image.py',
				'ImagePalette.py',
				'BmpImagePlugin.py',
				'ImageFile.py',
				'GifImagePlugin.py',
				'JpegImagePlugin.py',
				'PpmImagePlugin.py',
				'TiffImagePlugin.py',
				]
	EXCLUDES = 'dospath, posixpath, macpath, tk83, tcl83, _tkinter, pyHnj, _imagingtk, Tkinter, GimpPaletteFile, GimpGradientFile, PIL.ImageTk, FixTk, TkConstants, xmllib'
	INCLUDES='reportlab.platypus.paragraph, reportlab.lib.xmllib'
	for f in PIL_names:
		INCLUDES=INCLUDES+(',PIL.%s'%os.path.splitext(f)[0])
	DIRS=[]

	def force_rm(d):
		'destroy directory d'
		if os.path.isdir(d):
			for p in os.listdir(d):
				force_rm(os.path.join(d,p))
			os.rmdir(d)
		elif os.path.isfile(d):
			os.remove(d)

	DIRS=repr(DIRS)[1:-1]
	cfg='''[MYSINGLEFILE]
name=%(EXE)s
type=FULLEXE
userunw=0
zlib=MYZLIB
misc=%(MISC_DLLS)s
excludes=%(EXCLUDES)s
script=%(SCRIPT)s
debug=%(DEBUG)d

[MYZLIB]
type=PYZ
name=%(NAME)s.pyz
dependencies=%(SCRIPT)s
excludes=%(EXCLUDES)s
includes=%(INCLUDES)s
directories=%(DIRS)s
''' % globals()
	if not '-noclean' in sys.argv: force_rm(BUILD_DIR)
	if not os.path.exists(BUILD_DIR): os.mkdir(BUILD_DIR)
	if not os.path.exists(bpath(SCRIPT)):
		shutil.copyfile(SCRIPT, bpath(SCRIPT))
		open(bpath(CFG),'w').write(cfg)
	cwd = os.getcwd()
	force_rm(EXE)
	os.chdir(BUILD_DIR)
	force_rm(EXE)
	print("<<<<Freezing %(SCRIPT)s to %(EXE)s>>>>"%globals())
	rc = os.system('python%s %s %s %s' % (PDEBUG and "_d" or "",ipath('Builder.py'),DEBUG and "-d" or "",CFG))
	print("return code:", rc)
	DST = os.path.join(cwd,EXE)
	if os.path.isfile(EXE):
		shutil.copyfile(EXE,DST)
		force_rm(EXE)
		print('Moved %(EXE)s to %(cwd)s' % globals())



# vim: sw=4:expandtab:foldmethod=marker
#
# Copyright (c) 2006, Mathieu Fenniak
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# * Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# * The name of the author may not be used to endorse or promote products
# derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from .generic import *
from .pdf import PdfFileReader, PdfFileWriter
from .pagerange import PageRange
from sys import version_info
if version_info < ( 3, 0 ):
    from cStringIO import StringIO
else:
    from io import StringIO
    from io import FileIO as file

class _MergedPage(object):
    """
    _MergedPage is used internally by PdfFileMerger to collect necessary information on each page that is being merged.
    """
    def __init__(self, pagedata, src, id):
        self.src = src
        self.pagedata = pagedata
        self.out_pagedata = None
        self.id = id
        
class PdfFileMerger(object):
    """
    PdfFileMerger merges multiple PDFs into a single PDF. It can concatenate, 
    slice, insert, or any combination of the above.
    
    See the functions "merge" (or "append") and "write" (or "overwrite") for
    usage information.
    """
    
    def __init__(self, strict=True):
        """
        >>> PdfFileMerger()
        
        Initializes a PdfFileMerger, no parameters required
        """
        self.inputs = []
        self.pages = []
        self.output = PdfFileWriter()
        self.bookmarks = []
        self.named_dests = []
        self.id_count = 0
        self.strict = strict
        
    def merge(self, position, fileobj, bookmark=None, pages=None, import_bookmarks=True):
        """
        >>> merge(position, file, bookmark=None, pages=None, import_bookmarks=True)
        
        Merges the pages from the source document specified by "file" into the output
        file at the page number specified by "position".
        
        Optionally, you may specify a bookmark to be applied at the beginning of the 
        included file by supplying the text of the bookmark in the "bookmark" parameter.
        
        You may prevent the source document's bookmarks from being imported by
        specifying "import_bookmarks" as False.
        
        The optional "pages" parameter can be a PageRange or a 
        (start, stop[, step]) tuple to merge only the specified range of pages
        from the source document into the output document.
        """
        
        # This parameter is passed to self.inputs.append and means
        # that the stream used was created in this method.
        my_file = False
        
        # If the fileobj parameter is a string, assume it is a path
        # and create a file object at that location. If it is a file,
        # copy the file's contents into a StringIO stream object; if 
        # it is a PdfFileReader, copy that reader's stream into a 
        # StringIO stream.
        # If fileobj is none of the above types, it is not modified
        if type(fileobj) in (str, str):
            fileobj = file(fileobj, 'rb')
            my_file = True
        elif isinstance(fileobj, file):
            fileobj.seek(0)
            filecontent = fileobj.read()
            fileobj = StringIO(filecontent)
            my_file = True
        elif isinstance(fileobj, PdfFileReader):
            orig_tell = fileobj.stream.tell()   
            fileobj.stream.seek(0)
            filecontent = StringIO(fileobj.stream.read())
            fileobj.stream.seek(orig_tell) # reset the stream to its original location
            fileobj = filecontent
            my_file = True
            
        # Create a new PdfFileReader instance using the stream
        # (either file or StringIO) created above
        pdfr = PdfFileReader(fileobj, strict=self.strict)
        
        # Find the range of pages to merge.
        if pages == None:
            pages = (0, pdfr.getNumPages())
        elif isinstance(pages, PageRange):
            pages = pages.indices(pdfr.getNumPages())
        elif not isinstance(pages, tuple):
            raise TypeError('"pages" must be a tuple of (start, stop[, step])')
        
        srcpages = []
        if bookmark:
            bookmark = Bookmark(TextStringObject(bookmark), NumberObject(self.id_count), NameObject('/Fit'))
        
        outline = []
        if import_bookmarks:
            outline = pdfr.getOutlines()
            outline = self._trim_outline(pdfr, outline, pages)
        
        if bookmark:
            self.bookmarks += [bookmark, outline]
        else:
            self.bookmarks += outline
        
        dests = pdfr.namedDestinations
        dests = self._trim_dests(pdfr, dests, pages)
        self.named_dests += dests
        
        # Gather all the pages that are going to be merged
        for i in range(*pages):
            pg = pdfr.getPage(i)
            
            id = self.id_count
            self.id_count += 1
            
            mp = _MergedPage(pg, pdfr, id)
            
            srcpages.append(mp)

        self._associate_dests_to_pages(srcpages)
        self._associate_bookmarks_to_pages(srcpages)
            
        
        # Slice to insert the pages at the specified position
        self.pages[position:position] = srcpages
        
        # Keep track of our input files so we can close them later
        self.inputs.append((fileobj, pdfr, my_file))
        
        
    def append(self, fileobj, bookmark=None, pages=None, import_bookmarks=True):
        """
        >>> append(file, bookmark=None, pages=None, import_bookmarks=True):
        
        Identical to the "merge" function, but assumes you want to concatenate all pages
        onto the end of the file instead of specifying a position.
        """
        
        self.merge(len(self.pages), fileobj, bookmark, pages, import_bookmarks)
        
    
    def write(self, fileobj):
        """
        >>> write(file)
        
        Writes all data that has been merged to "file" (which can be a filename or any
        kind of file-like object)
        """
        my_file = False
        if type(fileobj) in (str, str):
            fileobj = file(fileobj, 'wb')
            my_file = True


        # Add pages to the PdfFileWriter
        # The commented out line below was replaced with the two lines below it to allow PdfFileMerger to work with PyPdf 1.13
        for page in self.pages:
            self.output.addPage(page.pagedata)
            page.out_pagedata = self.output.getReference(self.output._pages.getObject()["/Kids"][-1].getObject())
            #idnum = self.output._objects.index(self.output._pages.getObject()["/Kids"][-1].getObject()) + 1
            #page.out_pagedata = IndirectObject(idnum, 0, self.output)

        # Once all pages are added, create bookmarks to point at those pages
        self._write_dests()
        self._write_bookmarks()
        
        # Write the output to the file   
        self.output.write(fileobj)
        
        if my_file:
            fileobj.close()


        
    def close(self):
        """
        >>> close()
        
        Shuts all file descriptors (input and output) and clears all memory usage
        """
        self.pages = []
        for fo, pdfr, mine in self.inputs:
            if mine:
                fo.close()
        
        self.inputs = []
        self.output = None

    def addMetadata(self, infos):
        """See addMetadata method in PdfFileWriter class"""
        self.output.addMetadata(infos)
    
    def setPageLayout(self, layout):
        """See setPageLayout() methods in pdf.py"""
        self.output.setPageLayout(layout)

    def setPageMode(self, mode):
        """See setPageMode() methods in pdf.py"""
        self.output.setPageMode(mode)

    def _trim_dests(self, pdf, dests, pages):
        """
        Removes any named destinations that are not a part of the specified 
        page set.
        """
        new_dests = []
        prev_header_added = True
        for k, o in list(dests.items()):
            for j in range(*pages):
                if pdf.getPage(j).getObject() == o['/Page'].getObject():
                    o[NameObject('/Page')] = o['/Page'].getObject()
                    assert str(k) == str(o['/Title'])
                    new_dests.append(o)
                    break
        return new_dests
    
    def _trim_outline(self, pdf, outline, pages):
        """
        Removes any outline/bookmark entries that are not a part of the 
        specified page set.
        """
        new_outline = []
        prev_header_added = True
        for i, o in enumerate(outline):
            if isinstance(o, list):
                sub = self._trim_outline(pdf, o, pages)
                if sub:
                    if not prev_header_added:
                        new_outline.append(outline[i-1])
                    new_outline.append(sub)
            else:
                prev_header_added = False
                for j in range(*pages):
                    if pdf.getPage(j).getObject() == o['/Page'].getObject():
                        o[NameObject('/Page')] = o['/Page'].getObject()
                        new_outline.append(o)
                        prev_header_added = True
                        break
        return new_outline
   
    def _write_dests(self):
        dests = self.named_dests
        
        for v in dests:
            pageno = None
            pdf = None
            if '/Page' in v:
                for i, p in enumerate(self.pages):
                    if p.id == v['/Page']:
                        v[NameObject('/Page')] = p.out_pagedata
                        pageno = i
                        pdf = p.src
                        break
            if pageno != None:
                self.output.addNamedDestinationObject(v)
 
    def _write_bookmarks(self, bookmarks=None, parent=None):
        
        if bookmarks == None:
            bookmarks = self.bookmarks
        

        last_added = None
        for b in bookmarks:
            if isinstance(b, list):
                self._write_bookmarks(b, last_added)
                continue
                
            pageno = None
            pdf = None
            if '/Page' in b:
                for i, p in enumerate(self.pages):
                    if p.id == b['/Page']:
                        #b[NameObject('/Page')] = p.out_pagedata
                        args = [NumberObject(p.id), NameObject(b['/Type'])]
                        #nothing more to add
                        #if b['/Type'] == '/Fit' or b['/Type'] == '/FitB'
                        if b['/Type'] == '/FitH' or b['/Type'] == '/FitBH':
                            if '/Top' in b and not isinstance(b['/Top'], NullObject):
                                args.append(FloatObject(b['/Top']))
                            else:
                                args.append(FloatObject(0))
                            del b['/Top']
                        elif b['/Type'] == '/FitV' or b['/Type'] == '/FitBV':
                            if '/Left' in b and not isinstance(b['/Left'], NullObject):
                                args.append(FloatObject(b['/Left']))
                            else:
                                args.append(FloatObject(0))
                            del b['/Left']
                        elif b['/Type'] == '/XYZ':
                            if '/Left' in b and not isinstance(b['/Left'], NullObject):
                                args.append(FloatObject(b['/Left']))
                            else:
                                args.append(FloatObject(0))
                            if '/Top' in b and not isinstance(b['/Top'], NullObject):
                                args.append(FloatObject(b['/Top']))
                            else:
                                args.append(FloatObject(0))
                            if '/Zoom' in b and not isinstance(b['/Zoom'], NullObject):
                                args.append(FloatObject(b['/Zoom']))
                            else:
                                args.append(FloatObject(0))
                            del b['/Top'], b['/Zoom'], b['/Left']
                        elif b['/Type'] == '/FitR':
                            if '/Left' in b and not isinstance(b['/Left'], NullObject):
                                args.append(FloatObject(b['/Left']))
                            else:
                                args.append(FloatObject(0))
                            if '/Bottom' in b and not isinstance(b['/Bottom'], NullObject):
                                args.append(FloatObject(b['/Bottom']))
                            else:
                                args.append(FloatObject(0))
                            if '/Right' in b and not isinstance(b['/Right'], NullObject):
                                args.append(FloatObject(b['/Right']))
                            else:
                                args.append(FloatObject(0))
                            if '/Top' in b and not isinstance(b['/Top'], NullObject):
                                args.append(FloatObject(b['/Top']))
                            else:
                                args.append(FloatObject(0))
                            del b['/Left'], b['/Right'], b['/Bottom'], b['/Top']

                        b[NameObject('/A')] = DictionaryObject({NameObject('/S'): NameObject('/GoTo'), NameObject('/D'): ArrayObject(args)})
                       
                        pageno = i
                        pdf = p.src
                        break
            if pageno != None:
                del b['/Page'], b['/Type']
                last_added = self.output.addBookmarkDict(b, parent)    

    def _associate_dests_to_pages(self, pages):
        for nd in self.named_dests:
            pageno = None
            np = nd['/Page']
            
            if isinstance(np, NumberObject):
                continue
            
            for p in pages:
                if np.getObject() == p.pagedata.getObject():
                    pageno = p.id
            
            if pageno != None:
                nd[NameObject('/Page')] = NumberObject(pageno)
            else:
                raise ValueError("Unresolved named destination '%s'" % (nd['/Title'],))
    
    def _associate_bookmarks_to_pages(self, pages, bookmarks=None):
        if bookmarks == None:
            bookmarks = self.bookmarks

        for b in bookmarks:
            if isinstance(b, list):
                self._associate_bookmarks_to_pages(pages, b)
                continue
                
            pageno = None
            bp = b['/Page']
            
            if isinstance(bp, NumberObject):
                continue
                
            for p in pages:
                if bp.getObject() == p.pagedata.getObject():
                    pageno = p.id
            
            if pageno != None:
                b[NameObject('/Page')] = NumberObject(pageno)
            else:
                raise ValueError("Unresolved bookmark '%s'" % (b['/Title'],))
                
    def findBookmark(self, bookmark, root=None):
    	if root == None:
    		root = self.bookmarks
    	
    	for i, b in enumerate(root):
    		if isinstance(b, list):
    			res = self.findBookmark(bookmark, b)
    			if res:
    				return [i] + res
    		elif b == bookmark or b['/Title'] == bookmark:
    			return [i]
    
    	return None

    def addBookmark(self, title, pagenum, parent=None):
        """
        Add a bookmark to the pdf, using the specified title and pointing at 
        the specified page number. A parent can be specified to make this a
        nested bookmark below the parent.
        """

        if parent == None:
        	iloc = [len(self.bookmarks)-1]
        elif isinstance(parent, list):
        	iloc = parent
        else:
        	iloc = self.findBookmark(parent)
        
        dest = Bookmark(TextStringObject(title), NumberObject(pagenum), NameObject('/FitH'), NumberObject(826))
        
        if parent == None:
        	self.bookmarks.append(dest)
        else:
        	bmparent = self.bookmarks
        	for i in iloc[:-1]:
        		bmparent = bmparent[i]
        	npos = iloc[-1]+1
        	if npos < len(bmparent) and isinstance(bmparent[npos], list):
        		bmparent[npos].append(dest)
        	else:
        		bmparent.insert(npos, [dest])
        return dest
        		
        
    def addNamedDestination(self, title, pagenum):
        """
        Add a destination to the pdf, using the specified title and pointing
        at the specified page number.
        """
        
        dest = Destination(TextStringObject(title), NumberObject(pagenum), NameObject('/FitH'), NumberObject(826))
        self.named_dests.append(dest)


class OutlinesObject(list):
    def __init__(self, pdf, tree, parent=None):
        list.__init__(self)
        self.tree = tree
        self.pdf = pdf
        self.parent = parent
    
    def remove(self, index):
        obj = self[index]
        del self[index]
        self.tree.removeChild(obj)
        
    def add(self, title, page):
        pageRef = self.pdf.getObject(self.pdf._pages)['/Kids'][pagenum]
        action = DictionaryObject()
        action.update({
            NameObject('/D') : ArrayObject([pageRef, NameObject('/FitH'), NumberObject(826)]),
            NameObject('/S') : NameObject('/GoTo')
        })
        actionRef = self.pdf._addObject(action)
        bookmark = TreeObject()

        bookmark.update({
            NameObject('/A'): actionRef,
            NameObject('/Title'): createStringObject(title),
        })

        pdf._addObject(bookmark)

        self.tree.addChild(bookmark)
        
    def removeAll(self):
        for child in [x for x in self.tree.children()]:
            self.tree.removeChild(child)
            self.pop()



import re
import datetime
import decimal
from .generic import PdfObject
from xml.dom import getDOMImplementation
from xml.dom.minidom import parseString
from .utils import u_

RDF_NAMESPACE = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
DC_NAMESPACE = "http://purl.org/dc/elements/1.1/"
XMP_NAMESPACE = "http://ns.adobe.com/xap/1.0/"
PDF_NAMESPACE = "http://ns.adobe.com/pdf/1.3/"
XMPMM_NAMESPACE = "http://ns.adobe.com/xap/1.0/mm/"

# What is the PDFX namespace, you might ask?  I might ask that too.  It's
# a completely undocumented namespace used to place "custom metadata"
# properties, which are arbitrary metadata properties with no semantic or
# documented meaning.  Elements in the namespace are key/value-style storage,
# where the element name is the key and the content is the value.  The keys
# are transformed into valid XML identifiers by substituting an invalid
# identifier character with \u2182 followed by the unicode hex ID of the
# original character.  A key like "my car" is therefore "my\u21820020car".
#
# \u2182, in case you're wondering, is the unicode character
# \u{ROMAN NUMERAL TEN THOUSAND}, a straightforward and obvious choice for
# escaping characters.
#
# Intentional users of the pdfx namespace should be shot on sight.  A
# custom data schema and sensical XML elements could be used instead, as is
# suggested by Adobe's own documentation on XMP (under "Extensibility of
# Schemas").
#
# Information presented here on the /pdfx/ schema is a result of limited
# reverse engineering, and does not constitute a full specification.
PDFX_NAMESPACE = "http://ns.adobe.com/pdfx/1.3/"

iso8601 = re.compile("""
        (?P<year>[0-9]{4})
        (-
            (?P<month>[0-9]{2})
            (-
                (?P<day>[0-9]+)
                (T
                    (?P<hour>[0-9]{2}):
                    (?P<minute>[0-9]{2})
                    (:(?P<second>[0-9]{2}(.[0-9]+)?))?
                    (?P<tzd>Z|[-+][0-9]{2}:[0-9]{2})
                )?
            )?
        )?
        """, re.VERBOSE)

##
# An object that represents Adobe XMP metadata.
class XmpInformation(PdfObject):

    def __init__(self, stream):
        self.stream = stream
        docRoot = parseString(self.stream.getData())
        self.rdfRoot = docRoot.getElementsByTagNameNS(RDF_NAMESPACE, "RDF")[0]
        self.cache = {}

    def writeToStream(self, stream, encryption_key):
        self.stream.writeToStream(stream, encryption_key)

    def getElement(self, aboutUri, namespace, name):
        for desc in self.rdfRoot.getElementsByTagNameNS(RDF_NAMESPACE, "Description"):
            if desc.getAttributeNS(RDF_NAMESPACE, "about") == aboutUri:
                attr = desc.getAttributeNodeNS(namespace, name)
                if attr != None:
                    yield attr
                for element in desc.getElementsByTagNameNS(namespace, name):
                    yield element

    def getNodesInNamespace(self, aboutUri, namespace):
        for desc in self.rdfRoot.getElementsByTagNameNS(RDF_NAMESPACE, "Description"):
            if desc.getAttributeNS(RDF_NAMESPACE, "about") == aboutUri:
                for i in range(desc.attributes.length):
                    attr = desc.attributes.item(i)
                    if attr.namespaceURI == namespace:
                        yield attr
                for child in desc.childNodes:
                    if child.namespaceURI == namespace:
                        yield child

    def _getText(self, element):
        text = ""
        for child in element.childNodes:
            if child.nodeType == child.TEXT_NODE:
                text += child.data
        return text

    def _converter_string(value):
        return value

    def _converter_date(value):
        m = iso8601.match(value)
        year = int(m.group("year"))
        month = int(m.group("month") or "1")
        day = int(m.group("day") or "1")
        hour = int(m.group("hour") or "0")
        minute = int(m.group("minute") or "0")
        second = decimal.Decimal(m.group("second") or "0")
        seconds = second.to_integral(decimal.ROUND_FLOOR)
        milliseconds = (second - seconds) * 1000000
        tzd = m.group("tzd") or "Z"
        dt = datetime.datetime(year, month, day, hour, minute, seconds, milliseconds)
        if tzd != "Z":
            tzd_hours, tzd_minutes = [int(x) for x in tzd.split(":")]
            tzd_hours *= -1
            if tzd_hours < 0:
                tzd_minutes *= -1
            dt = dt + datetime.timedelta(hours=tzd_hours, minutes=tzd_minutes)
        return dt
    _test_converter_date = staticmethod(_converter_date)

    def _getter_bag(namespace, name, converter):
        def get(self):
            cached = self.cache.get(namespace, {}).get(name)
            if cached:
                return cached
            retval = []
            for element in self.getElement("", namespace, name):
                bags = element.getElementsByTagNameNS(RDF_NAMESPACE, "Bag")
                if len(bags):
                    for bag in bags:
                        for item in bag.getElementsByTagNameNS(RDF_NAMESPACE, "li"):
                            value = self._getText(item)
                            value = converter(value)
                            retval.append(value)
            ns_cache = self.cache.setdefault(namespace, {})
            ns_cache[name] = retval
            return retval
        return get

    def _getter_seq(namespace, name, converter):
        def get(self):
            cached = self.cache.get(namespace, {}).get(name)
            if cached:
                return cached
            retval = []
            for element in self.getElement("", namespace, name):
                seqs = element.getElementsByTagNameNS(RDF_NAMESPACE, "Seq")
                if len(seqs):
                    for seq in seqs:
                        for item in seq.getElementsByTagNameNS(RDF_NAMESPACE, "li"):
                            value = self._getText(item)
                            value = converter(value)
                            retval.append(value)
                else:
                    value = converter(self._getText(element))
                    retval.append(value)
            ns_cache = self.cache.setdefault(namespace, {})
            ns_cache[name] = retval
            return retval
        return get

    def _getter_langalt(namespace, name, converter):
        def get(self):
            cached = self.cache.get(namespace, {}).get(name)
            if cached:
                return cached
            retval = {}
            for element in self.getElement("", namespace, name):
                alts = element.getElementsByTagNameNS(RDF_NAMESPACE, "Alt")
                if len(alts):
                    for alt in alts:
                        for item in alt.getElementsByTagNameNS(RDF_NAMESPACE, "li"):
                            value = self._getText(item)
                            value = converter(value)
                            retval[item.getAttribute("xml:lang")] = value
                else:
                    retval["x-default"] = converter(self._getText(element))
            ns_cache = self.cache.setdefault(namespace, {})
            ns_cache[name] = retval
            return retval
        return get

    def _getter_single(namespace, name, converter):
        def get(self):
            cached = self.cache.get(namespace, {}).get(name)
            if cached:
                return cached
            value = None
            for element in self.getElement("", namespace, name):
                if element.nodeType == element.ATTRIBUTE_NODE:
                    value = element.nodeValue
                else:
                    value = self._getText(element)
                break
            if value != None:
                value = converter(value)
            ns_cache = self.cache.setdefault(namespace, {})
            ns_cache[name] = value
            return value
        return get

    ##
    # Contributors to the resource (other than the authors).  An unsorted
    # array of names.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_contributor = property(_getter_bag(DC_NAMESPACE, "contributor", _converter_string))

    ##
    # Text describing the extent or scope of the resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_coverage = property(_getter_single(DC_NAMESPACE, "coverage", _converter_string))

    ##
    # A sorted array of names of the authors of the resource, listed in order
    # of precedence.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_creator = property(_getter_seq(DC_NAMESPACE, "creator", _converter_string))

    ##
    # A sorted array of dates (datetime.datetime instances) of signifigance to
    # the resource.  The dates and times are in UTC.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_date = property(_getter_seq(DC_NAMESPACE, "date", _converter_date))

    ##
    # A language-keyed dictionary of textual descriptions of the content of the
    # resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_description = property(_getter_langalt(DC_NAMESPACE, "description", _converter_string))

    ##
    # The mime-type of the resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_format = property(_getter_single(DC_NAMESPACE, "format", _converter_string))

    ##
    # Unique identifier of the resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_identifier = property(_getter_single(DC_NAMESPACE, "identifier", _converter_string))

    ##
    # An unordered array specifying the languages used in the resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_language = property(_getter_bag(DC_NAMESPACE, "language", _converter_string))

    ##
    # An unordered array of publisher names.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_publisher = property(_getter_bag(DC_NAMESPACE, "publisher", _converter_string))

    ##
    # An unordered array of text descriptions of relationships to other
    # documents.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_relation = property(_getter_bag(DC_NAMESPACE, "relation", _converter_string))

    ##
    # A language-keyed dictionary of textual descriptions of the rights the
    # user has to this resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_rights = property(_getter_langalt(DC_NAMESPACE, "rights", _converter_string))

    ##
    # Unique identifier of the work from which this resource was derived.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_source = property(_getter_single(DC_NAMESPACE, "source", _converter_string))

    ##
    # An unordered array of descriptive phrases or keywrods that specify the
    # topic of the content of the resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_subject = property(_getter_bag(DC_NAMESPACE, "subject", _converter_string))

    ##
    # A language-keyed dictionary of the title of the resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_title = property(_getter_langalt(DC_NAMESPACE, "title", _converter_string))

    ##
    # An unordered array of textual descriptions of the document type.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    dc_type = property(_getter_bag(DC_NAMESPACE, "type", _converter_string))

    ##
    # An unformatted text string representing document keywords.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    pdf_keywords = property(_getter_single(PDF_NAMESPACE, "Keywords", _converter_string))

    ##
    # The PDF file version, for example 1.0, 1.3.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    pdf_pdfversion = property(_getter_single(PDF_NAMESPACE, "PDFVersion", _converter_string))

    ##
    # The name of the tool that created the PDF document.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    pdf_producer = property(_getter_single(PDF_NAMESPACE, "Producer", _converter_string))

    ##
    # The date and time the resource was originally created.  The date and
    # time are returned as a UTC datetime.datetime object.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    xmp_createDate = property(_getter_single(XMP_NAMESPACE, "CreateDate", _converter_date))
    
    ##
    # The date and time the resource was last modified.  The date and time
    # are returned as a UTC datetime.datetime object.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    xmp_modifyDate = property(_getter_single(XMP_NAMESPACE, "ModifyDate", _converter_date))

    ##
    # The date and time that any metadata for this resource was last
    # changed.  The date and time are returned as a UTC datetime.datetime
    # object.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    xmp_metadataDate = property(_getter_single(XMP_NAMESPACE, "MetadataDate", _converter_date))

    ##
    # The name of the first known tool used to create the resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    xmp_creatorTool = property(_getter_single(XMP_NAMESPACE, "CreatorTool", _converter_string))

    ##
    # The common identifier for all versions and renditions of this resource.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    xmpmm_documentId = property(_getter_single(XMPMM_NAMESPACE, "DocumentID", _converter_string))

    ##
    # An identifier for a specific incarnation of a document, updated each
    # time a file is saved.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    xmpmm_instanceId = property(_getter_single(XMPMM_NAMESPACE, "InstanceID", _converter_string))

    def custom_properties(self):
        if not hasattr(self, "_custom_properties"):
            self._custom_properties = {}
            for node in self.getNodesInNamespace("", PDFX_NAMESPACE):
                key = node.localName
                while True:
                    # see documentation about PDFX_NAMESPACE earlier in file
                    idx = key.find(u_("\u2182"))
                    if idx == -1:
                        break
                    key = key[:idx] + chr(int(key[idx+1:idx+5], base=16)) + key[idx+5:]
                if node.nodeType == node.ATTRIBUTE_NODE:
                    value = node.nodeValue
                else:
                    value = self._getText(node)
                self._custom_properties[key] = value
        return self._custom_properties

    ##
    # Retrieves custom metadata properties defined in the undocumented pdfx
    # metadata schema.
    # <p>Stability: Added in v1.12, will exist for all future v1.x releases.
    # @return Returns a dictionary of key/value items for custom metadata
    # properties.
    custom_properties = property(custom_properties)





__all__ = ('solve',)
def solve(func, target, x0=1.0, precision=1e-008, maxIterations=100, x1=None, failHard=1):
    """Finds x such that f(x)==target.
    To quote Newton & Raphson:  Given an equation, 
            f(x) = 0
        and an initial approximation, x(0),
        In Newton's method a better approximation is given by: 
            x(i+1) = x(i) - f(x(i)) / f'(x(i))
        where f'(x) is the first derivative of f, df/dx.
    In the Raphson adaptation we use an approximation to
    the differential which is given by
        deltaF/deltaX = (f(x+dx)-f(x))/dx
    """
    iters = 0
    f0 = func(x0)
    if failHard:
        b = x0
        bf = abs(f0-target)
    if x1 is None:
        if x0 == 0.0:
            x1 = 1.0  # otherwise a zero would kill ts
        else:
            x1 = x0 * 1.1
    while 1:
        if iters>=maxIterations:
            if failHard:
                assert iters < maxIterations, "Failed to converge after %s tries" % maxIterations
            else:
                return b
        f1 = func(x1)
        d = f1-target
        ad = abs(d)
        if failHard and ad<bf:
            b = x1
            bf = ad
        # if solved to RELATIVE precision, return now
        if ad/max(abs(target),1.0)<precision: return x1
        x2 = x1 - d/((f1-f0)/(x1-x0)) 
        x0 = x1
        f0 = f1
        x1 = x2
        iters += 1

def _test():
    def f(x):
        v = x*x
        return x<0 and -v or v
    for target in [-1, 1, 0]:
        x = solve(f,target)
        print('target=%f x=%f f(x)=%f' % (target,x,f(x)))

if __name__=='__main__': #noruntests
    _test()



#copyright ReportLab Europe Limited. 2000-2012
SEP_OPS='''
%%BeginResource: procset sep_ops 1.03 0
%%Title: (Separation Procs)
%%Version: 1.03 0
userdict /sep_ops 50 dict dup begin put
/bdef {bind def} bind def
/xdef {exch def} bdef
/colorimagebuffer { % helper proc called by customcolorimage
	0 1 2 index length 1 sub {
	dup 2 index exch get 255 exch sub 2 index 3 1 roll put
	}for
}bdef

/addprocs { % {proc1} {proc2} addprocs {{proc1}exec {proc2} exec}
	[ 3 1 roll
	/exec load
	dup 3 1 roll
	] cvx
} bdef

/L1? {
	/languagelevel where {
	pop languagelevel 2 lt
	}{
		true
	} ifelse
} bdef

/colorexists { % tests to see if printing on color device
	statusdict /processcolors known {
		statusdict /processcolors get exec
	}{ % processcolors not present
		/deviceinfo where { % check for dps environment
			pop deviceinfo /Colors known {
				deviceinfo /Colors get % get color value from DPS
				statusdict /processcolors {% add processcolors entry
					deviceinfo /Colors known {
						deviceinfo /Colors get
						}{
					1
				} ifelse
			} put
		}{
		1
	} ifelse
}{ % not in dps environment, assume monochrome
1
} ifelse
} ifelse
1 gt % return true for color devices, false for B&W
} bdef
/MakeReadOnlyArray { % size => [array]
/packedarray where {
pop packedarray
}{
array astore readonly
} ifelse
} bdef
/findcmykcustomcolor where {
pop
}{
/findcmykcustomcolor {% c m y k name findcmykcustomcolor array
5 MakeReadOnlyArray
} bdef
} ifelse
/setoverprint where {
pop
}{
/setoverprint {% boolean setoverprint -
pop
} bdef
} ifelse
/setcustomcolor where {
pop
}{
L1? {
/setcustomcolor { % array tint setcustomcolor -
exch
aload pop pop
4 { 4 index mul 4 1 roll } repeat
5 -1 roll pop
setcmykcolor
} bdef
}{
/setcustomcolor { % customcolorarray tint
exch
[ exch /Separation exch dup 4 get exch /DeviceCMYK exch
0 4 getinterval
[ exch /dup load exch cvx {mul exch dup}
/forall load /pop load dup] cvx
] setcolorspace setcolor
} bdef
} ifelse
} ifelse
% initialize variables to avoid unintentional early binding
/ik 0 def /iy 0 def /im 0 def /ic 0 def
/imagetint {% converts cmyk to grayscale equiv w/red book formula
% called by setcmykcolor and customcolorimage procs.
ic .3 mul
im .59 mul
iy .11 mul
ik add add add dup
1 gt{pop 1}if
} bdef
/setcmykcolor where {
pop
}{
% setcmykcolor not supported, call setgray instead
/setcmykcolor { % c m y k setcmykcolor --
/ik xdef /iy xdef /im xdef /ic xdef
imagetint
1 exch sub setgray
} bdef
} ifelse
/customcolorimage where {
pop
}{
L1? {
/customcolorimage{ % w h bps matrix proc array
gsave
colorexists {
aload pop pop
/ik xdef /iy xdef /im xdef /ic xdef
currentcolortransfer
{ik mul ik sub 1 add} addprocs
4 1 roll {iy mul iy sub 1 add} addprocs
4 1 roll{im mul im sub 1 add} addprocs
4 1 roll{ic mul ic sub 1 add} addprocs
4 1 roll setcolortransfer
/magentabuf 0 string def
/yellowbuf 0 string def
/blackbuf 0 string def
{
colorimagebuffer dup length magentabuf length ne{
dup length dup dup
/magentabuf exch string def
/yellowbuf exch string def
/blackbuf exch string def
}if
dup magentabuf copy yellowbuf copy
blackbuf copy pop
} addprocs
{magentabuf}{yellowbuf}{blackbuf} true 4 colorimage
}{ % non-color device
aload pop pop /ik xdef /iy xdef /im xdef /ic xdef
/tint imagetint def
currenttransfer
{tint mul 1 tint sub add} addprocs settransfer image
}ifelse
grestore
} bdef
}{ % Level 2 environment
/customcolorimage { % w h bps matrix proc array
gsave
[ exch /Separation exch dup 4 get exch /DeviceCMYK exch
0 4 getinterval
[ exch /dup load exch cvx {mul exch dup}
/forall load /pop load dup] cvx
] setcolorspace
10 dict begin
/ImageType 1 def
/DataSource exch def
/ImageMatrix exch def
/BitsPerComponent exch def
/Height exch def
/Width exch def
/Decode [1 0] def
currentdict end
image
grestore
} bdef
} ifelse
} ifelse
/setseparationgray where {
pop
}{
L1? {
/setseparationgray {
1 exch sub dup dup dup setcmykcolor
} bdef
}{
/setseparationgray {
[/Separation /All /DeviceCMYK
{dup dup dup}] setcolorspace 1 exch sub setcolor
} bdef
} ifelse
} ifelse
/separationimage where {
pop
}{
/separationimage {
gsave
1 1 1 1 (All)
findcmykcustomcolor customcolorimage
grestore
} bdef
} ifelse
currentdict readonly pop end
%%EndResource
'''



from io import StringIO

import sys
import unittest2

hasInstallHandler = hasattr(unittest2, 'installHandler')

class Test_TestProgram(unittest2.TestCase):

    # Horrible white box test
    def testNoExit(self):
        result = object()
        test = object()

        class FakeRunner(object):
            def run(self, test):
                self.test = test
                return result

        runner = FakeRunner()

        oldParseArgs = unittest2.TestProgram.parseArgs
        def restoreParseArgs():
            unittest2.TestProgram.parseArgs = oldParseArgs
        unittest2.TestProgram.parseArgs = lambda *args: None
        self.addCleanup(restoreParseArgs)

        def removeTest():
            del unittest2.TestProgram.test
        unittest2.TestProgram.test = test
        self.addCleanup(removeTest)

        program = unittest2.TestProgram(testRunner=runner, exit=False, verbosity=2)

        self.assertEqual(program.result, result)
        self.assertEqual(runner.test, test)
        self.assertEqual(program.verbosity, 2)

    class FooBar(unittest2.TestCase):
        def testPass(self):
            assert True
        def testFail(self):
            assert False

    class FooBarLoader(unittest2.TestLoader):
        """Test loader that returns a suite containing FooBar."""
        def loadTestsFromModule(self, module):
            return self.suiteClass(
                [self.loadTestsFromTestCase(Test_TestProgram.FooBar)])


    def test_NonExit(self):
        program = unittest2.main(exit=False,
                                argv=["foobar"],
                                testRunner=unittest2.TextTestRunner(stream=StringIO()),
                                testLoader=self.FooBarLoader())
        self.assertTrue(hasattr(program, 'result'))


    def test_Exit(self):
        self.assertRaises(
            SystemExit,
            unittest2.main,
            argv=["foobar"],
            testRunner=unittest2.TextTestRunner(stream=StringIO()),
            exit=True,
            testLoader=self.FooBarLoader())


    def test_ExitAsDefault(self):
        self.assertRaises(
            SystemExit,
            unittest2.main,
            argv=["foobar"],
            testRunner=unittest2.TextTestRunner(stream=StringIO()),
            testLoader=self.FooBarLoader())


class InitialisableProgram(unittest2.TestProgram):
    exit = False
    result = None
    verbosity = 1
    defaultTest = None
    testRunner = None
    testLoader = unittest2.defaultTestLoader
    progName = 'test'
    test = 'test'
    def __init__(self, *args):
        pass

RESULT = object()

class FakeRunner(object):
    initArgs = None
    test = None
    raiseError = False

    def __init__(self, **kwargs):
        FakeRunner.initArgs = kwargs
        if FakeRunner.raiseError:
            FakeRunner.raiseError = False
            raise TypeError

    def run(self, test):
        FakeRunner.test = test
        return RESULT

class TestCommandLineArgs(unittest2.TestCase):
    
    def setUp(self):
        self.program = InitialisableProgram()
        self.program.createTests = lambda: None
        FakeRunner.initArgs = None
        FakeRunner.test = None
        FakeRunner.raiseError = False
    
    def testHelpAndUnknown(self):
        program = self.program
        def usageExit(msg=None):
            program.msg = msg
            program.exit = True
        program.usageExit = usageExit
        
        for opt in '-h', '-H', '--help':
            program.exit = False
            program.parseArgs([None, opt])
            self.assertTrue(program.exit)
            self.assertIsNone(program.msg)
    
        program.parseArgs([None, '-$'])
        self.assertTrue(program.exit)
        self.assertIsNotNone(program.msg)
    
    def testVerbosity(self):
        program = self.program
        
        for opt in '-q', '--quiet':
            program.verbosity = 1
            program.parseArgs([None, opt])
            self.assertEqual(program.verbosity, 0)

        for opt in '-v', '--verbose':
            program.verbosity = 1
            program.parseArgs([None, opt])
            self.assertEqual(program.verbosity, 2)

    def testBufferCatchFailfast(self):
        program = self.program
        for arg, attr in (('buffer', 'buffer'), ('failfast', 'failfast'),
                      ('catch', 'catchbreak')):
            if attr == 'catch' and not hasInstallHandler:
                continue
            
            short_opt = '-%s' % arg[0]
            long_opt = '--%s' % arg
            for opt in short_opt, long_opt:
                setattr(program, attr, None)
                
                program.parseArgs([None, opt])
                self.assertTrue(getattr(program, attr))

            for opt in short_opt, long_opt:
                not_none = object()
                setattr(program, attr, not_none)
                
                program.parseArgs([None, opt])
                self.assertEqual(getattr(program, attr), not_none)

    def testRunTestsRunnerClass(self):
        program = self.program
        
        program.testRunner = FakeRunner
        program.verbosity = 'verbosity'
        program.failfast = 'failfast'
        program.buffer = 'buffer'
        
        program.runTests()
        
        self.assertEqual(FakeRunner.initArgs, {'verbosity': 'verbosity', 
                                                'failfast': 'failfast',
                                                'buffer': 'buffer'})
        self.assertEqual(FakeRunner.test, 'test')
        self.assertIs(program.result, RESULT)

    def testRunTestsRunnerInstance(self):
        program = self.program
        
        program.testRunner = FakeRunner()
        FakeRunner.initArgs = None
        
        program.runTests()

        # A new FakeRunner should not have been instantiated
        self.assertIsNone(FakeRunner.initArgs)
        
        self.assertEqual(FakeRunner.test, 'test')
        self.assertIs(program.result, RESULT)

    def testRunTestsOldRunnerClass(self):
        program = self.program
        
        FakeRunner.raiseError = True
        program.testRunner = FakeRunner
        program.verbosity = 'verbosity'
        program.failfast = 'failfast'
        program.buffer = 'buffer'
        program.test = 'test'
        
        program.runTests()
        
        # If initialising raises a type error it should be retried
        # without the new keyword arguments
        self.assertEqual(FakeRunner.initArgs, {})
        self.assertEqual(FakeRunner.test, 'test')
        self.assertIs(program.result, RESULT)
    
    def testCatchBreakInstallsHandler(self):
        module = sys.modules['unittest2.main']
        original = module.installHandler
        def restore():
            module.installHandler = original
        self.addCleanup(restore)

        self.installed = False
        def fakeInstallHandler():
            self.installed = True
        module.installHandler = fakeInstallHandler
        
        program = self.program
        program.catchbreak = True
        
        program.testRunner = FakeRunner
        
        program.runTests()
        self.assertTrue(self.installed)


if __name__ == '__main__':
    unittest2.main()



#copyright ReportLab Inc. 2000-2012
#see license.txt for license details
__version__=''' $Id$ '''
__all__=('tt2xml',)
from xml.sax.saxutils import escape, quoteattr
from reportlab.lib.utils import isBytes, isUnicode, isStr
def tt2xml(tt):
    '''convert tuple tree form to unicode xml'''
    if tt is None: return ''
    if isBytes(tt):
        return tt2xml(tt.decode('utf8'))
    if isUnicode(tt):
        return escape(tt)
    if isinstance(tt,list):
        return ''.join(tt2xml(x) for x in tt)
    if isinstance(tt,tuple):
        tag = tt[0].decode('utf8')
        L=['<'+tag].append
        C = tt[2]
        if tt[1]:
            for k,v in tt[1].items():
                L((' %s=%s' % (k,quoteattr(v))).decode('utf8'))
        if C is not None:
            L('>')
            L(tt2xml(C))
            L('</'+tag+'>')
        else:
            L('/>')
        return ''.join(L.__self__)
    raise ValueError('Invalid value %r passed to tt2xml' % tt)

def findTags(tt, tname, tags=None):
    '''looks for a named tag in a pyRXP tuple tree (t)
    returns R where R is  a list of items (X) where items are either tuple 
    trees (t) where the tag is found or the items can be a list of t[X]
    R = [X*]
    X = t | [t[X]]
    '''
    if tags is None: tags=[]
    if isStr(tt) or tt is None:
        return tags
    if isinstance(tt, tuple):
        if tt[0]==tname:
            T1 = findTags(tt[2],tname,[])
            if T1:
                tags.append([tt,T1])
            else:
                tags.append(tt)
            return tags
        else:
            return findTags(tt[2],tname,tags)
    if isinstance(tt, list):
        for x in tt:
            findTags(x, tname, tags)
        return tags
    raise ValueError('invalid argument for tt=%r' % tt)



#Autogenerated by ReportLab guiedit do not edit
from rlextra.graphics.quickchart import quickChart, ALLOWED_CHART_TYPES
from reportlab.lib.colors import CMYKColor, toColor, Color

def QCTestDrawing(**kw):
	return quickChart(**kw)

def makeDrawing(html,msg,**kw):
	L = []
	for k,v in list(kw.items()):
		L.append('%s=%s' % (k,repr(v)))
	if msg and L:
		msg += ' '
	msg2 = 'quickchart(%s)' % ','.join(L)
	try:
		qc = QCTestDrawing(**kw)
		for k,v in list(kw.items()):
			L.append('%s=%s' % (k,repr(v)))
		import md5
		h = md5.md5()
		h.update(msg+msg2)
		fnRoot = 'QCTestDrawing' + h.hexdigest()
		qc.save(formats=['pdf','gif','eps'],outDir='pmout',fnRoot=fnRoot)
		if not msg:
			print("%s\n"% (msg2))
			html.append('<h2>%s</h2><img src="%s.gif">\n' % (msg2,fnRoot))
		else:
			print("%s\n%s\n"% (msg, msg2))
			html.append('<h2>%s</h2><h3>%s</h3><img src="%s.gif">\n' % (msg,msg2,fnRoot))
	except:
		msg = "!!! FAILED ON CHART '%s' !!!" % msg
		if msg2:
			html.append('<h2><font color="red">%s<br/>%s</font></h2>\n' % (msg, msg2))
			print("%s\n%s\n"% (msg, msg2))
		else:
			html.append('<h2><font color="red">%s</font></h2>\n' % msg)
			print("%s\n%s\n"% (msg))
		import sys
		tbt,tbv,tb = sys.exc_info()
		from rlextra.utils.log import _tb2html
		html.append(_tb2html(tbt,tbv,tb))

def main():
	LP = tuple(enumerate((None,'top','top_right','right','bottom_right','bottom','bottom_left','left','top_left')))
	HLEGS = tuple(enumerate((None,False,True)))
	LEGENDMODES = tuple(enumerate((None,'fullWidth','fullHeight')))
	ALLOWED_CHART_TYPES.sort()
	html = []
	for chartType in ALLOWED_CHART_TYPES:
		makeDrawing(html, '',   chartType=chartType)

	makeDrawing(html,"0.1 Line chart should have both x and y gridlines",chartType='linechart',yAxisVisible=1, yAxisGridLines=1, xAxisVisible=1, xAxisGridLines=1)
	makeDrawing(html,"0.11 Line plot should have both x and y gridlines",chartType='lineplot',yAxisVisible=1, yAxisGridLines=1, xAxisVisible=1, xAxisGridLines=1, data=[[1.5,3,4.5],[1,2,3],[1.5,2.5,3.5]])
	makeDrawing(html,"0.12 0.1+yAxisLabelTextFormat='$%(decfmt(value*10000,2,'.',''))s'",chartType='linechart',yAxisVisible=1, yAxisGridLines=1, xAxisVisible=1, xAxisGridLines=1, yAxisLabelTextFormat= "$%(decfmt(value*10000,2,'.',''))s")
	makeDrawing(html,"0.13 0.1+yAxisLabelTextFormat='$%(decfmt(value*10000,-2,'.',','))s'",chartType='linechart',yAxisVisible=1, yAxisGridLines=1, xAxisVisible=1, xAxisGridLines=1, yAxisLabelTextFormat= "$%(decfmt(value*10000,-2,'.',','))s")
	makeDrawing(html,"0.14 0.1+yAxisLabelTextFormat='$%(decfmt(value*10000,(1 if notAllInt else 0),'.',','))s'",chartType='linechart',yAxisVisible=1, yAxisGridLines=1, xAxisVisible=1, xAxisGridLines=1, yAxisLabelTextFormat= "$%(decfmt(value*10000,(1 if notAllInt else 0),'.',','))s")
	makeDrawing(html,"0.15 0.1+yAxisLabelTextFormat='$%(decfmt(value*10000,2,',','.'))s'",chartType='linechart',yAxisVisible=1, yAxisGridLines=1, xAxisVisible=1, xAxisGridLines=1, yAxisLabelTextFormat= "$%(decfmt(value*10000,2,',','.'))s")

	makeDrawing(html,'1.1 Clustered_bar with the yAxisVisible and yAxisGridlines on',chartType='clustered_bar',yAxisVisible=1, yAxisGridLines=1, xAxisVisible=0)
	makeDrawing(html,'1.2 Stacked_bar with the yAxisVisible and yAxisGridlines on',chartType='stacked_bar',yAxisVisible=1, yAxisGridLines=1, xAxisVisible=0)

	makeDrawing(html,'1.3 Clustered_bar with the xAxisVisible and xAxisGridlines on',chartType='clustered_bar',xAxisVisible=1, xAxisGridLines=1, yAxisVisible=0)
	makeDrawing(html,'1.4 Stacked_bar with the xAxisVisible and xAxisGridlines on',chartType='stacked_bar',xAxisVisible=1, xAxisGridLines=1, yAxisVisible=0)

	makeDrawing(html,'2.1 Clustered_bar with both the x Axis and y Axis invisible - should resize correctly',chartType='stacked_bar',xAxisVisible=0, yAxisVisible=0,showBoundaries=1)
	makeDrawing(html,'2.2 Stacked_bar with both the x Axis and y Axis invisible - should resize correctly',chartType='clustered_bar',xAxisVisible=0, yAxisVisible=0,showBoundaries=1)

	makeDrawing(html,'3.1 Stacked_bar with dataLabelsType set to None',chartType='stacked_bar',dataLabelsType=None)
	makeDrawing(html,'3.2 Clustered_bar with dataLabelsType set to None',chartType='clustered_bar',dataLabelsType=None)
	makeDrawing(html,'3.3 Pie with dataLabelsType set to None',chartType='pie',dataLabelsType=None)

	makeDrawing(html,"3.4 Stacked_bar with dataLabelsType set to 'values'",chartType='stacked_bar',dataLabelsType='values')
	makeDrawing(html,"3.5 Clustered_bar with dataLabelsType set to 'values'",chartType='clustered_bar',dataLabelsType='values')
	makeDrawing(html,"3.6 Pie with dataLabelsType set to 'values'",chartType='pie',dataLabelsType='values')

	makeDrawing(html,"3.7 Stacked_bar with dataLabelsType set to 'percent'",chartType='stacked_bar',dataLabelsType='percent')
	makeDrawing(html,"3.8 Clustered_bar with dataLabelsType set to 'percent'",chartType='clustered_bar',dataLabelsType='percent')
	makeDrawing(html,"3.9 Pie with dataLabelsType set to 'percent'",chartType='pie',dataLabelsType='percent')
	makeDrawing(html,"3.71 Stacked_bar with dataLabelsType set to 'percent,2'",chartType='stacked_bar',dataLabelsType='percent,2')
	makeDrawing(html,"3.81 Clustered_bar with dataLabelsType set to 'percent,2'",chartType='clustered_bar',dataLabelsType='percent,2')
	makeDrawing(html,"3.91 Pie with dataLabelsType set to 'percent,2'",chartType='pie',dataLabelsType='percent,2')
	makeDrawing(html,"3.72 Stacked_bar with dataLabelsType set to '%(percent).1f%%'",chartType='stacked_bar',dataLabelsType='%(percent).1f%%')
	makeDrawing(html,"3.82 Clustered_bar with dataLabelsType set to '%(percent).2f%%'",chartType='clustered_bar',dataLabelsType='%(percent).2f%%')
	makeDrawing(html,"3.92 Pie with dataLabelsType set to '%(percent).3f%% per Annum'",chartType='pie',dataLabelsType='%(percent).3f%% per Annum')
	makeDrawing(html,"3.73 Stacked_bar with dataLabelsType set to '$%(decfmt(value*10000,2,'.',''))s'",chartType='stacked_bar',dataLabelsType='$%(decfmt(value*10000,2,\'.\',\'\'))s')
	makeDrawing(html,"3.74 Stacked_bar with dataLabelsType set to '$%(decfmt(value*10000,2,'.',','))s'",chartType='stacked_bar',dataLabelsType='$%(decfmt(value*10000,2,\'.\',\',\'))s')
	makeDrawing(html,"3.83 Clustered_bar with dataLabelsType set to '$%(decfmt(value*14000,2,',','.'))s'",chartType='clustered_bar',dataLabelsType='$%(decfmt(value*14000,1,\',\',\'.\'))s')
	makeDrawing(html,"3.93 Pie with dataLabelsType set to '$%(decfmt(value*12000,2))s/Year'",chartType='pie',dataLabelsType='$%(decfmt(value*12000,2))s/Year')

	makeDrawing(html,"4.1 Pie with dataLabelsType unset - no datalabels should be printed",chartType='pie')
	makeDrawing(html,"4.2 Pie with dataLabelsType set to 'values' - values should be used for dataLabels",chartType='pie',dataLabelsType='values')
	makeDrawing(html,"4.3 Pie with dataLabelsType set to 'percent' - percentages should be used for dataLabels",chartType='pie',dataLabelsType='percent')
	makeDrawing(html,"4.4 Pie with with category names - set dataLabelsType '%(category)s'",chartType='pie',dataLabelsType='%(category)s',categoryNames=['A','B','C','D'])

	makeDrawing(html,"4.5.0 Pie with overlap",chartType='pie',dataLabelsType='%(percent).3f%% per Annum',data=[0.9,1.1,2.2,40,57],categoryNames=['0.9','1.1','2.2','40','57'])
	makeDrawing(html,"4.5.1 Pie with overlap orderMode='alternate'",chartType='pie',orderMode='alternate',dataLabelsType='%(percent).3f%% per Annum',data=[0.9,1.1,2.2,40,57],categoryNames=['0.9','1.1','2.2','40','57'])
	makeDrawing(html,"4.5.2 Pie with overlap checkLabelOverlap=1",chartType='pie',checkLabelOverlap=1,dataLabelsType='%(percent).3f%% per Annum',data=[0.9,1.1,2.2,40,57],categoryNames=['0.9','1.1','2.2','40','57'])
	makeDrawing(html,"4.5.3 Pie with overlap orderMode='alternate' checkLabelOverlap=1",chartType='pie',checkLabelOverlap=1,orderMode='alternate', dataLabelsType='%(percent).3f%% per Annum',data=[0.9,1.1,2.2,40,57],categoryNames=['0.9','1.1','2.2','40','57'])

	makeDrawing(html,"5.1 Title should be (ascent * 1.5) from the font base line ",chartType='pie', titleText="This is the Title")

	makeDrawing(html,"6.1 Bar with integer data - axis should display without decimals",chartType='bar', data=[[100, 200, 300, 400]], xAxisVisible=1, yAxisVisible=1)
	makeDrawing(html,"6.2 Column with integer data - axis should display without decimals",chartType='column', data=[[100, 200, 300, 400]], xAxisVisible=1, yAxisVisible=1)
	makeDrawing(html,"6.3 Bar with floating point data - axis should display with decimals",chartType='bar', data=[[0.01, 0.02, 0.03, 0.04]], xAxisVisible=1, yAxisVisible=1)
	makeDrawing(html,"6.4 Bar with floating point data - axis should display with decimals",chartType='column', data=[[0.01, 0.02, 0.03, 0.04]], xAxisVisible=1, yAxisVisible=1)

	makeDrawing(html,"7.1 x Axis and y Axis gridlines should be the same width",chartType='bar', xAxisVisible=1, yAxisVisible=1, xAxisGridLines=1, yAxisGridLines=1)
	makeDrawing(html,"7.2 x Axis and y Axis gridlines should be the same width",chartType='column', xAxisVisible=1, yAxisVisible=1, xAxisGridLines=1, yAxisGridLines=1)

	makeDrawing(html,"8.1 When using data = [[120,20]], the value axis should no longer show negative values of (-50,0,50,100,150)",chartType='column', data=[[120,20]])
	makeDrawing(html,"8.1a When using data = [[120,20]], the value axis should no longer show negative values of (-50,0,50,100,150)",chartType='bar', data=[[120,20]])
	makeDrawing(html,"8.2 When using negative data the gridline of the category axis should be correctly sized",chartType='column', data=[[-120,-20,20]], xAxisGridLines=1)
	makeDrawing(html,"8.2a When using negative data the gridline of the category axis should be correctly sized",chartType='bar', data=[[-120,-20,20]], yAxisGridLines=1)

	for k,legendMode in LEGENDMODES:
		for j,hleg in HLEGS:
			for i,p in LP:
				makeDrawing(html,"9.1.%d.%d.%d Blue border round background, no fill on background legendPos=%s hleg=%s legendMode=%s" % (k,j,i,p,hleg,legendMode),chartType='column', bgColor = None, bgStrokeColor='blue', seriesNames=('A long series name','Another very long series name'),legendPos=p,hleg=hleg,legendMaxWFrac=0.5,legendMaxHFrac=0.5,legendMode=legendMode)
	makeDrawing(html,"9.2 Blue border round background, yellow fill on background",chartType='bar', bgColor = 'yellow', bgStrokeColor='blue')

	makeDrawing(html,"10.1 Test piechart with data of '[[230,340]]' and dataLabelsType of 'percent' (as per Nardi's test)", chartType='pie',data=[[230,340]],categoryNames=['category1','category2'],seriesNames=[],bgColor=None,plotColor=CMYKColor(0,0,1,0),legendPos='left',legendFontName='Helvetica',legendFontColor=CMYKColor(0,0,0,2),titleText='This is the main title',titleFontName='Helvetica-Bold',titleFontSize=18,titleFontColor=CMYKColor(0,1,1,1),dataLabelsType='percent',dataLabelsFontName='Helvetica',dataLabelsFontSize=14,dataLabelsFontColor=CMYKColor(0,1,1,0))
	for k,legendMode in LEGENDMODES:
		for j,hleg in HLEGS:
			for i,p in LP:
				makeDrawing(html,"10.2.%d.%d.%d piechart with more than 10 slices legendPos=%s hleg=%s legendMode=%s"%(k,j,i,p,hleg,legendMode), chartType = 'exploded_pie',data = [[27.00000,3.00000,10.00000,5.00000,5.00000,15.00000,35.00000,12,17,11,19,23,32]],categoryNames = ['Category 1','Category 2','Category 3','Category 4','Category 5','Category 6','Category 7','Cat 8','Cat 9','Cat 10','Cat 11','Cat 12','Cat 13'],seriesNames = [],chartColors = [CMYKColor(0.00,1.00,0.00,0.00),CMYKColor(1.00,1.00,0.00,0.00),CMYKColor(0.00,0.00,1.00,0.00),CMYKColor(0.00,1.00,1.00,0.00),CMYKColor(0.00,0.80,1.00,0.00),CMYKColor(0.00,0.40,1.00,0.00),CMYKColor(0.80,0.00,1.00,0.00),toColor('red'),toColor('grey'),toColor('brown'),toColor('magenta'),toColor('darkblue'),toColor('pink')],bgColor = None,plotColor = None,legendPos = p,legendFontName = 'Helvetica',legendFontSize = 9.00,legendFontColor = CMYKColor(0.05,0.45,1.00,0.00),titleText = '',titleFontName = 'Helvetica-Bold',titleFontSize = 14.00,titleFontColor = CMYKColor(0.00,0.00,0.00,1.00),dataLabelsType = '%.1f',dataLabelsFontName = 'Helvetica',dataLabelsFontSize = 9.00,dataLabelsFontColor = CMYKColor(0.05,0.45,1.00,0.00),width = 401.54384,height = 150.86375,hleg=hleg,legendMaxWFrac=0.5,legendMaxHFrac=0.5,legendMode=legendMode)

	makeDrawing(html,"11.1 Test for black lines in 3D exploded piechart", chartType = 'exploded_pie3d',data = [[27.00000,3.00000,10.00000,5.00000,5.00000,15.00000,35.00000]],categoryNames = ['Category 1','Category 2','Category 3','Category 4','Category 5','Category 6','Category 7'],seriesNames = [],chartColors = [CMYKColor(0.00,1.00,0.00,0.00),CMYKColor(1.00,1.00,0.00,0.00),CMYKColor(0.00,0.00,1.00,0.00),CMYKColor(0.00,1.00,1.00,0.00),CMYKColor(0.00,0.80,1.00,0.00),CMYKColor(0.00,0.40,1.00,0.00),CMYKColor(0.80,0.00,1.00,0.00)],bgColor = None,plotColor = None,legendPos = 'right',legendFontName = 'Helvetica',legendFontSize = 9.00,legendFontColor = CMYKColor(0.05,0.45,1.00,0.00),titleText = '',titleFontName = 'Helvetica-Bold',titleFontSize = 14.00,titleFontColor = CMYKColor(0.00,0.00,0.00,1.00),dataLabelsType = '%.1f',dataLabelsFontName = 'Helvetica',dataLabelsFontSize = 9.00,dataLabelsFontColor = CMYKColor(0.05,0.45,1.00,0.00),width = 401.54384,height = 150.86375)
	for k,legendMode in LEGENDMODES:
		for j,hleg in HLEGS:
			for i,p in LP:
				makeDrawing(html,"11.2.%d.%d.%d 3D exploded piechart with more than 10 slices legendPos=%s hleg=%s legendMode=%s"%(k,j,i,p,hleg,legendMode), chartType = 'exploded_pie3d',data = [[27.00000,3.00000,10.00000,5.00000,5.00000,15.00000,35.00000,12,17,11,19,23,32]],categoryNames = ['Category 1','Category 2','Category 3','Category 4','Category 5','Category 6','Category 7','Cat 8','Cat 9','Cat 10','Cat 11','Cat 12','Cat 13'],seriesNames = [],chartColors = [CMYKColor(0.00,1.00,0.00,0.00),CMYKColor(1.00,1.00,0.00,0.00),CMYKColor(0.00,0.00,1.00,0.00),CMYKColor(0.00,1.00,1.00,0.00),CMYKColor(0.00,0.80,1.00,0.00),CMYKColor(0.00,0.40,1.00,0.00),CMYKColor(0.80,0.00,1.00,0.00),toColor('red'),toColor('grey'),toColor('brown'),toColor('magenta'),toColor('darkblue'),toColor('pink')],bgColor = None,plotColor = None,legendPos = p,legendFontName = 'Helvetica',legendFontSize = 9.00,legendFontColor = CMYKColor(0.05,0.45,1.00,0.00),titleText = '',titleFontName = 'Helvetica-Bold',titleFontSize = 14.00,titleFontColor = CMYKColor(0.00,0.00,0.00,1.00),dataLabelsType = '%.1f',dataLabelsFontName = 'Helvetica',dataLabelsFontSize = 9.00,dataLabelsFontColor = CMYKColor(0.05,0.45,1.00,0.00),width = 401.54384,height = 150.86375,hleg=hleg,legendMaxWFrac=0.5,legendMaxHFrac=0.5,legendMode=legendMode)
	makeDrawing(html,"11.211 3D exploded piechart with more than 10 slices legend at top legendMaxHFrac=0.125", chartType = 'exploded_pie3d',legendMaxHFrac=0.125,data = [[27.00000,3.00000,10.00000,5.00000,5.00000,15.00000,35.00000,12,17,11,19,23,32]],categoryNames = ['Category 1','Category 2','Category 3','Category 4','Category 5','Category 6','Category 7','Cat 8','Cat 9','Cat 10','Cat 11','Cat 12','Cat 13'],seriesNames = [],chartColors = [CMYKColor(0.00,1.00,0.00,0.00),CMYKColor(1.00,1.00,0.00,0.00),CMYKColor(0.00,0.00,1.00,0.00),CMYKColor(0.00,1.00,1.00,0.00),CMYKColor(0.00,0.80,1.00,0.00),CMYKColor(0.00,0.40,1.00,0.00),CMYKColor(0.80,0.00,1.00,0.00),toColor('red'),toColor('grey'),toColor('brown'),toColor('magenta'),toColor('darkblue'),toColor('pink')],bgColor = None,plotColor = None,legendPos = 'top',legendFontName = 'Helvetica',legendFontSize = 9.00,legendFontColor = CMYKColor(0.05,0.45,1.00,0.00),titleText = '',titleFontName = 'Helvetica-Bold',titleFontSize = 14.00,titleFontColor = CMYKColor(0.00,0.00,0.00,1.00),dataLabelsType = '%.1f',dataLabelsFontName = 'Helvetica',dataLabelsFontSize = 9.00,dataLabelsFontColor = CMYKColor(0.05,0.45,1.00,0.00),width = 401.54384,height = 150.86375)
	makeDrawing(html,"11.212 3D exploded piechart with more than 10 slices legend at top legendMaxHFrac=0.09", chartType = 'exploded_pie3d',legendMaxHFrac=0.09,data = [[27.00000,3.00000,10.00000,5.00000,5.00000,15.00000,35.00000,12,17,11,19,23,32]],categoryNames = ['Category 1','Category 2','Category 3','Category 4','Category 5','Category 6','Category 7','Cat 8','Cat 9','Cat 10','Cat 11','Cat 12','Cat 13'],seriesNames = [],chartColors = [CMYKColor(0.00,1.00,0.00,0.00),CMYKColor(1.00,1.00,0.00,0.00),CMYKColor(0.00,0.00,1.00,0.00),CMYKColor(0.00,1.00,1.00,0.00),CMYKColor(0.00,0.80,1.00,0.00),CMYKColor(0.00,0.40,1.00,0.00),CMYKColor(0.80,0.00,1.00,0.00),toColor('red'),toColor('grey'),toColor('brown'),toColor('magenta'),toColor('darkblue'),toColor('pink')],bgColor = None,plotColor = None,legendPos = 'top',legendFontName = 'Helvetica',legendFontSize = 9.00,legendFontColor = CMYKColor(0.05,0.45,1.00,0.00),titleText = '',titleFontName = 'Helvetica-Bold',titleFontSize = 14.00,titleFontColor = CMYKColor(0.00,0.00,0.00,1.00),dataLabelsType = '%.1f',dataLabelsFontName = 'Helvetica',dataLabelsFontSize = 9.00,dataLabelsFontColor = CMYKColor(0.05,0.45,1.00,0.00),width = 401.54384,height = 150.86375)
	makeDrawing(html,"11.213 3D exploded piechart with more than 10 slices legend at top legendMaxWFrac=0.8", chartType = 'exploded_pie3d',legendMaxWFrac=0.8,data = [[27.00000,3.00000,10.00000,5.00000,5.00000,15.00000,35.00000,12,17,11,19,23,32]],categoryNames = ['Category 1','Category 2','Category 3','Category 4','Category 5','Category 6','Category 7','Cat 8','Cat 9','Cat 10','Cat 11','Cat 12','Cat 13'],seriesNames = [],chartColors = [CMYKColor(0.00,1.00,0.00,0.00),CMYKColor(1.00,1.00,0.00,0.00),CMYKColor(0.00,0.00,1.00,0.00),CMYKColor(0.00,1.00,1.00,0.00),CMYKColor(0.00,0.80,1.00,0.00),CMYKColor(0.00,0.40,1.00,0.00),CMYKColor(0.80,0.00,1.00,0.00),toColor('red'),toColor('grey'),toColor('brown'),toColor('magenta'),toColor('darkblue'),toColor('pink')],bgColor = None,plotColor = None,legendPos = 'top',legendFontName = 'Helvetica',legendFontSize = 9.00,legendFontColor = CMYKColor(0.05,0.45,1.00,0.00),titleText = '',titleFontName = 'Helvetica-Bold',titleFontSize = 14.00,titleFontColor = CMYKColor(0.00,0.00,0.00,1.00),dataLabelsType = '%.1f',dataLabelsFontName = 'Helvetica',dataLabelsFontSize = 9.00,dataLabelsFontColor = CMYKColor(0.05,0.45,1.00,0.00),width = 401.54384,height = 150.86375)
	makeDrawing(html,"12.1 xAxisLabelAngle=-70",chartType='linechart', dataLabelsType='values', categoryNames=['Oranges','Lemons','Apples','Pears'], xAxisLabelAngle=-70)
	makeDrawing(html,"12.2 yAxisLabelAngle=-70",chartType='linechart', dataLabelsType='values', categoryNames=['Oranges','Lemons','Apples','Pears'], yAxisLabelAngle=-70)
	makeDrawing(html,"12.3 xAxisLabelAngle=70",chartType='linechart', dataLabelsType='values', categoryNames=['Oranges','Lemons','Apples','Pears'], xAxisLabelAngle=70)
	makeDrawing(html,"12.3 yAxisLabelAngle=70",chartType='linechart', dataLabelsType='values', categoryNames=['Oranges','Lemons','Apples','Pears'], yAxisLabelAngle=70)
	makeDrawing(html,"13.1 textData", chartType='linechart', dataLabelsType='values', textData="A B C D\na b c d")
	makeDrawing(html,"13.2 textData", chartType='column', dataLabelsType='values', textData="A B C D\na b c d")
	makeDrawing(html,"13.3 textData", chartType='bar', dataLabelsType='values', textData="A B C D\na b c d")
	makeDrawing(html,"14.1 Ron Error Empty Chart", chartType = 'clustered_column',data =[[0],[0],[0]],categoryNames = ['Oct'],seriesNames = ['Series 1','Series  2','Series 3'],chartColors =  [CMYKColor(0.00,0.20,0.34,0.00),CMYKColor(0.00,0.00,0.40,0.00),CMYKColor (0.20,0.00,0.09,0.00)],bgColor = None,plotColor = None,legendPos =	'right',legendFontName = 'Helvetica',legendFontColor =	CMYKColor(0.00,0.00,0.00,1.00),titleText = 'Banzai!',titleFontName =  'Helvetica-Bold',titleFontColor =  CMYKColor(0.00,0.00,0.00,1.00),xTitleText = '',xTitleFontName =  'Helvetica',xTitleFontColor = CMYKColor(0.00,0.00,0.00,1.00),yTitleText  = '',yTitleFontName = 'Helvetica',yTitleFontColor =	CMYKColor(0.00,0.00,0.00,1.00),xAxisVisible = 1,xAxisLabelAngle =  30,xAxisFontName = 'Helvetica',xAxisFontColor =	CMYKColor(0.00,0.00,0.00,1.00),yAxisVisible = 1,yAxisFontName =  'Helvetica',yAxisFontColor =  CMYKColor(0.00,0.00,0.00,1.00),xAxisGridLines = 0,yAxisGridLines =  1,dataLabelsType = None,width = 366.00000,height = 347.50000)
	makeDrawing(html,"14.2 Liron Axis error",chartType = 'linechart',data = [[0.00,0.00,0.00,0.00,-1.07]],categoryNames = ['a','b','c','d','e'],seriesNames = ['Series 1'],chartColors = [Color(0.82,0,0),Color(0.32,0,0),Color(0.32,0.16,0.06),Color(0.32,0.16,0.06),Color(0.32,0.16,0.06)],bgColor = None,plotColor = None,legendPos = 'right',legendFontName = 'Times-Roman',legendFontSize = 12.00,legendFontColor = Color(0.00,0.00,0.00),titleText = '',titleFontName = 'Times-Roman',titleFontSize = 12.00,titleFontColor = Color(0.00,0.00,0.00),xTitleText = '',xTitleFontName = 'Times-Roman',xTitleFontSize = 12.00,xTitleFontColor = Color(0.00,0.00,0.00),yTitleText = '',yTitleFontName = 'Times-Roman',yTitleFontSize = 12.00,yTitleFontColor = Color(0.00,0.00,0.00),xAxisVisible = 1,xAxisLabelAngle = 30,xAxisFontName = 'Times-Roman',xAxisFontSize = 12.00,xAxisFontColor = Color(0.00,0.00,0.00),yAxisVisible = 1,yAxisFontName = 'Times-Roman',yAxisFontSize = 12.00,yAxisFontColor = Color(0.00,0.00,0.00),xAxisGridLines = 0,yAxisGridLines = 0,dataLabelsType = None,width = 431.00,height = 343.00)
	import os, sys
	fn = os.path.join('pmout', 'qctests.html')
	open(fn,'w').write('<html><head></head><body>%s</body></html>' % ('<hr width=100%>'.join(html)))
	if sys.platform=='mac':
		from reportlab.lib.utils import markfilename
		markfilename(fn,ext='HTML')

if __name__=="__main__": #NORUNTESTS
	main()



"mymodule.py implements example plug-in functionality for rml2pdf"

# <plugInGraphic module="mymodule" function="myfunction">example data</plugInGraphic>

def myfunction(canv, data):
    print("myfunction", (canv, data))
    canv.line(0,0, 100, 100)

# <plugInFlowable module="mymodule" function="myFlowable">this data</plugInFlowable>

# note: this is wrapped in a "real" flowable so it doesn't need to inherit from platypus.Flowable
class myFlowable:
    def __init__(self, data=''):
        print("myFlowable init", data)
    def wrap(self, w, h):
        print("myFlowable wrap", w, h)
        self.width = w
        self.height = h
        return (0,0)
    def draw(self):
        canv = self.canv
        canv.saveState()
        canv.setFillColorRGB(1,0,0)
        canv.rect(0,0,self.width,self.height,fill=1,stroke=0)
        self.canv.line(0,0,100,0)
        canv.restoreState()
        print("myFlowable drew a line", (0,0,100,0))

from reportlab.platypus.xpreformatted import XPreformatted
from reportlab.platypus.flowables import Flowable
from reportlab.lib.styles import getSampleStyleSheet
class myPreformatted(XPreformatted):
    """This is used by test_009_splitting under the test directory. It
    has been placed here to make sure that it is available in any
    standard installation of RML."""
    def __init__(self, text=None, style=getSampleStyleSheet()["Normal"],bulletText = None, dedent=0, frags=None):
        if text is None: text = """This page tests splitting - this is a pluginFlowable from
mymodule.py, using XPreformatted a reportlab style 'Normal' . It
should split between the two frames on this page. These frames should
be ON A NEW PAGE, otherwise this test has FAILED. This page tests
splitting - this XPreformatted should split between the two frames on
this page. This page tests splitting - this XPreformatted should split
between the two frames on this page. This page tests splitting - this
XPreformatted should split between the two frames on this page. This
page tests splitting - this XPreformatted should split between the two
frames on this page."""
        XPreformatted.__init__(self,text,style,bulletText,frags,dedent)

class linkURL(Flowable):
    def __init__(self, URL, text=None):
        self._URL = URL
        self._text = text or URL

    def wrap(self, w, h):
        c = self.canv
        fs = c._fontsize
        w = c.stringWidth(self._text,c._fontname,fs)
        self._width, self._height = w, 1.2*fs
        return (w,self._height)

    def draw(self):
        c = self.canv
        w, h = self._width, self._height
        c.drawString(0,h/6.,self._text)
        c.linkURL(self._URL,(0,0,w,h),relative=1,color=None,dashArray=None)

def _getArgs(*args,**kw):
    return args, kw

def _evalArgs(data):
    if data[0]!='(': data = '(%s)' % data
    return eval('_getArgs'+data)

def linkRect(canv,data):
    args, kw = _evalArgs(data)
    canv.linkRect(*args,**kw)

def bookmarkPage(canv,name):
    canv.bookmarkPage(name)

def _toColor(s):
    from reportlab.lib.colors import toColor
    if s in ['','none','None',None]:
        return None
    try:
        return toColor(s)
    except:
        raise ValueError('Bad color value %r' % s)

def symbols(canv,data):
    '''
    produces filled symbols

    arguments
    n   number of points default 5
    x,y     initial symbol location
    dx      delta x for each new symbol
    dy      delta y for each symbol
    name    symbol name (suitable for makeMarker)
    fillColor   symbol fill colour
    strokeColor symbol stroke colour
    strokeWidth symbol stroke width
    angle   symbol angle
    '''
    args, kw = _evalArgs(data)
    if args:
        raise TypeError('Symbols takes no positional arguments')
    params = dict(
            n=5,
            x='72',
            y='72',
            dx='None',
            dy='0',
            name='StarFive',
            fillColor='yellow',
            strokeColor=None,
            strokeWidth='1',
            angle=0,
            )
    params.update(kw)

    from reportlab.graphics.shapes import Drawing
    from reportlab.graphics.widgets.markers import makeMarker
    from rlextra.rml2pdf.rml2pdf import readLengthOrNone

    name = params['name']
    n = int(params['n'])
    x0 = readLengthOrNone(params['x'])
    y0 = readLengthOrNone(params['y'])
    angle = float(params['angle'])
    size = readLengthOrNone(params['size'])
    strokeWidth = readLengthOrNone(params['strokeWidth'])
    dx = readLengthOrNone(params['dx'])
    if dx is None: dx = 2+1.5*size
    dy = readLengthOrNone(params['dy'])
    strokeColor = _toColor(params['strokeColor'])
    fillColor = _toColor(params['fillColor'])

    D = Drawing()
    for i in range(n):
        m = makeMarker(name)
        m.x = dx*i
        m.y = dy*i
        m.angle = angle
        m.size = size
        m.strokeWidth = strokeWidth
        m.strokeColor = strokeColor
        m.fillColor = fillColor
        D.add(m)

    D.drawOn(canv,x0,y0)

_Index=None
def getIndex(style=None, dot=None, tableStyle=None, indexClass=None, offset=0, format='123'):
    global _Index
    if not _Index:
        _Index = indexClass(style=style, dot=dot, tableStyle=tableStyle, offset=offset, format=format)
    return _Index

try:
    import pickle as pickle
except ImportError:
    import pickle
import base64
class IndexStarter(Flowable):
    def __init__(self, style=None, dot=None, tableStyle=None, indexClass='SimpleIndex', format='123', offset=0):
        self.style = style
        self.dot = dot
        self.tableStyle = tableStyle
        self.format = format
        self.offset = int(offset)

        def importFrom(mod, cls):
            return getattr(__import__(mod, globals(), locals(), [cls]), cls)

        try:
            self.indexClass = importFrom('reportlab.platypus.tableofcontents', indexClass)
        except (ImportError, AttributeError):
            mod, indexClass = indexClass.rsplit('.', 1)
            self.indexClass = importFrom(mod, indexClass)

    def wrap(self, w, h):
        rmlcontext = self._doctemplateAttr('_nameSpace')['rmlcontext']
        if rmlcontext:
            def checkStyle(s):
                try:
                    return rmlcontext.names[s]
                except KeyError:
                    raise KeyError('Style %s not found in IndexStarter.' % s)
            def getStyle(s,tAllowed=False):
                if not s: return None
                if tAllowed and isinstance(s,tuple):
                    return tuple(map(checkStyle,s))
                return checkStyle(s)
            style = getStyle(self.style,1)
            tableStyle = getStyle(self.tableStyle)
        _Index = getIndex(style, self.dot, tableStyle, self.indexClass, offset=self.offset,format=self.format)
        _Index.clearEntries()
        def _indexAdd(canv,kind,label):
            _Index(canv,kind,base64.encodestring(pickle.dumps((label,None,None))).strip())
        self.canv._indexAdd=_indexAdd
        return 0,0

    def draw(self):
        pass

class IndexFinisher(Flowable):
    _init = True
    def __init__(self):
        if self._init:
            x = getIndex()
            self.__dict__ = x.__dict__
            self.__class__ = x.__class__
            self._init = False
